@@ -1,4 +1,4 @@
-/* Copyright (c) 2017 - 2020 LiteSpeed Technologies Inc.  See LICENSE. */
+/* Copyright (c) 2017 - 2021 LiteSpeed Technologies Inc.  See LICENSE. */
 #ifndef __LSQUIC_H__
 #define __LSQUIC_H__
 
@@ -24,8 +24,8 @@ extern "C" {
 #endif
 
 #define LSQUIC_MAJOR_VERSION 2
-#define LSQUIC_MINOR_VERSION 12
-#define LSQUIC_PATCH_VERSION 14
+#define LSQUIC_MINOR_VERSION 27
+#define LSQUIC_PATCH_VERSION 6
 
 /**
  * Engine flags:
@@ -76,16 +76,26 @@ enum lsquic_version
 #define LSQUIC_EXPERIMENTAL_Q098 0
 #endif
 
-    /**
-     * IETF QUIC Draft-25
-     */
-    LSQVER_ID25,
-
     /**
      * IETF QUIC Draft-27
      */
     LSQVER_ID27,
 
+    /**
+     * IETF QUIC Draft-28; this version is deprecated.
+     */
+    LSQVER_ID28,
+
+    /**
+     * IETF QUIC Draft-29
+     */
+    LSQVER_ID29,
+
+    /**
+     * IETF QUIC Draft-32
+     */
+    LSQVER_ID32,
+
     /**
      * Special version to trigger version negotiation.
      * [draft-ietf-quic-transport-11], Section 3.
@@ -96,7 +106,8 @@ enum lsquic_version
 };
 
 /**
- * We currently support versions 43, 46, 50, Draft-25, and Draft-27.
+ * We currently support versions 43, 46, 50, Draft-27, Draft-28, Draft-29,
+ * and Draft-32.
  * @see lsquic_version
  */
 #define LSQUIC_SUPPORTED_VERSIONS ((1 << N_LSQVER) - 1)
@@ -109,15 +120,17 @@ enum lsquic_version
 #define LSQUIC_EXPERIMENTAL_VERSIONS ( \
                             (1 << LSQVER_VERNEG) | LSQUIC_EXPERIMENTAL_Q098)
 
-#define LSQUIC_DEPRECATED_VERSIONS 0
+#define LSQUIC_DEPRECATED_VERSIONS ((1 << LSQVER_ID27) | (1 << LSQVER_ID28))
 
 #define LSQUIC_GQUIC_HEADER_VERSIONS (1 << LSQVER_043)
 
-#define LSQUIC_IETF_VERSIONS ((1 << LSQVER_ID25) | (1 << LSQVER_ID27) \
-                                                    | (1 << LSQVER_VERNEG))
+#define LSQUIC_IETF_VERSIONS ((1 << LSQVER_ID27) | (1 << LSQVER_ID28) \
+                          | (1 << LSQVER_ID29) \
+                          | (1 << LSQVER_ID32) | (1 << LSQVER_VERNEG))
 
-#define LSQUIC_IETF_DRAFT_VERSIONS ((1 << LSQVER_ID25) | (1 << LSQVER_ID27) \
-                                                    | (1 << LSQVER_VERNEG))
+#define LSQUIC_IETF_DRAFT_VERSIONS ((1 << LSQVER_ID27) | (1 << LSQVER_ID28) \
+                                  | (1 << LSQVER_ID29) \
+                                  | (1 << LSQVER_ID32) | (1 << LSQVER_VERNEG))
 
 enum lsquic_hsk_status
 {
@@ -126,18 +139,18 @@ enum lsquic_hsk_status
      */
     LSQ_HSK_FAIL,
     /**
-     * The handshake succeeded without 0-RTT.
+     * The handshake succeeded without session resumption.
      */
     LSQ_HSK_OK,
     /**
-     * The handshake succeeded with 0-RTT.
+     * The handshake succeeded with session resumption.
      */
-    LSQ_HSK_0RTT_OK,
+    LSQ_HSK_RESUMED_OK,
     /**
-     * The handshake failed because of 0-RTT (early data rejected).  Retry
-     * the connection without 0-RTT.
+     * Session resumption failed.  Retry the connection without session
+     * resumption.
      */
-    LSQ_HSK_0RTT_FAIL,
+    LSQ_HSK_RESUMED_FAIL,
 };
 
 /**
@@ -174,13 +187,16 @@ struct lsquic_stream_if {
     void (*on_read)     (lsquic_stream_t *s, lsquic_stream_ctx_t *h);
     void (*on_write)    (lsquic_stream_t *s, lsquic_stream_ctx_t *h);
     void (*on_close)    (lsquic_stream_t *s, lsquic_stream_ctx_t *h);
+    /* Called when datagram is ready to be written */
+    ssize_t (*on_dg_write)(lsquic_conn_t *c, void *, size_t);
+    /* Called when datagram is read from a packet.  This callback is required
+     * when es_datagrams is true.  Take care to process it quickly, as this
+     * is called during lsquic_engine_packet_in().
+     */
+    void (*on_datagram)(lsquic_conn_t *, const void *buf, size_t);
     /* This callback in only called in client mode */
     /**
-     * When handshake is completed, this callback is called.  `ok' is set
-     * to true if handshake was successful; otherwise, `ok' is set to
-     * false.
-     *
-     * This callback is optional.
+     * When handshake is completed, this optional callback is called.
      */
     void (*on_hsk_done)(lsquic_conn_t *c, enum lsquic_hsk_status s);
     /**
@@ -191,16 +207,37 @@ struct lsquic_stream_if {
                                                         size_t token_size);
     /**
      * This optional callback lets client record information needed to
-     * perform a zero-RTT handshake next time around.
+     * perform a session resumption next time around.
      */
-    void (*on_zero_rtt_info)(lsquic_conn_t *c, const unsigned char *, size_t);
+    void (*on_sess_resume_info)(lsquic_conn_t *c, const unsigned char *, size_t);
+    /**
+     * Optional callback is called as soon as the peer resets a stream.
+     * The argument `how' is either 0, 1, or 2, meaning "read", "write", and
+     * "read and write", respectively (just like in shutdown(2)).  This
+     * signals the user to stop reading, writing, or both.
+     *
+     * Note that resets differ in gQUIC and IETF QUIC.  In gQUIC, `how' is
+     * always 2; in IETF QUIC, `how' is either 0 or 1 because one can reset
+     * just one direction in IETF QUIC.
+     */
+    void (*on_reset)    (lsquic_stream_t *s, lsquic_stream_ctx_t *h, int how);
+    /**
+     * Optional callback is called when a CONNECTION_CLOSE frame is received.
+     * This allows the application to log low-level diagnostic information about
+     * errors received with the CONNECTION_CLOSE frame. If app_error is -1 then
+     * it is considered unknown if this is an app_error or not.
+     */
+    void (*on_conncloseframe_received)(lsquic_conn_t *c,
+                                       int app_error, uint64_t error_code,
+                                       const char *reason, int reason_len);
 };
 
 struct ssl_ctx_st;
 struct ssl_st;
+struct lsxpack_header;
 
 /**
- * QUIC engine in server role needs access to certificates.  This is
+ * QUIC engine in server mode needs access to certificates.  This is
  * accomplished by providing a callback and a context to the engine
  * constructor.
  */
@@ -318,6 +355,9 @@ typedef struct ssl_ctx_st * (*lsquic_lookup_cert_f)(
 #define LSQUIC_DF_QPACK_ENC_MAX_BLOCKED 100
 #define LSQUIC_DF_QPACK_ENC_MAX_SIZE 4096
 
+/* By default, QPACK experiments are turned off */
+#define LSQUIC_DF_QPACK_EXPERIMENT 0
+
 /** ECN is disabled by default */
 #define LSQUIC_DF_ECN 0
 
@@ -330,17 +370,75 @@ typedef struct ssl_ctx_st * (*lsquic_lookup_cert_f)(
 /** Turn spin bit on by default */
 #define LSQUIC_DF_SPIN 1
 
-/** Turn off delayed ACKs extension by default */
-#define LSQUIC_DF_DELAYED_ACKS 0
+/** Turn on delayed ACKs extension by default */
+#define LSQUIC_DF_DELAYED_ACKS 1
+
+/**
+ * Defaults for the Packet Tolerance PID Controller (PTPC) used by the
+ * Delayed ACKs extension:
+ */
+#define LSQUIC_DF_PTPC_PERIODICITY 3
+#define LSQUIC_DF_PTPC_MAX_PACKTOL 150
+#define LSQUIC_DF_PTPC_DYN_TARGET 1
+#define LSQUIC_DF_PTPC_TARGET 1.0
+#define LSQUIC_DF_PTPC_PROP_GAIN 0.8
+#define LSQUIC_DF_PTPC_INT_GAIN 0.35
+#define LSQUIC_DF_PTPC_ERR_THRESH 0.05
+#define LSQUIC_DF_PTPC_ERR_DIVISOR 0.05
 
 /** Turn on timestamp extension by default */
 #define LSQUIC_DF_TIMESTAMPS 1
 
-/* 1: Cubic; 2: BBR */
-#define LSQUIC_DF_CC_ALGO 1
+/* Use Adaptive CC by default */
+#define LSQUIC_DF_CC_ALGO 3
+
+/* Default value of the CC RTT threshold is 1.5 ms */
+#define LSQUIC_DF_CC_RTT_THRESH 1500
+
+/** Turn off datagram extension by default */
+#define LSQUIC_DF_DATAGRAMS 0
+
+/** Assume optimistic NAT by default. */
+#define LSQUIC_DF_OPTIMISTIC_NAT 1
+
+/** Turn on Extensible HTTP Priorities by default. */
+#define LSQUIC_DF_EXT_HTTP_PRIO 1
 
 /** By default, incoming packet size is not limited. */
-#define LSQUIC_DF_MAX_PACKET_SIZE_RX 0
+#define LSQUIC_DF_MAX_UDP_PAYLOAD_SIZE_RX 0
+
+/**
+ * By default, greasing the QUIC bit is enabled (if peer sent
+ * the "grease_quic_bit" transport parameter).
+ */
+#define LSQUIC_DF_GREASE_QUIC_BIT 1
+
+/** By default, DPLPMTUD is enabled */
+#define LSQUIC_DF_DPLPMTUD 1
+
+/** By default, this value is left up to the engine. */
+#define LSQUIC_DF_BASE_PLPMTU 0
+
+/** By default, this value is left up to the engine. */
+#define LSQUIC_DF_MAX_PLPMTU 0
+
+/** By default, drop no-progress connections after 60 seconds on the server */
+#define LSQUIC_DF_NOPROGRESS_TIMEOUT_SERVER 60
+
+/** By default, do not use no-progress timeout on the client */
+#define LSQUIC_DF_NOPROGRESS_TIMEOUT_CLIENT 0
+
+/** By default, we use the minimum timer of 1000 milliseconds */
+#define LSQUIC_DF_MTU_PROBE_TIMER 1000
+
+/** By default, calling on_close() is not delayed */
+#define LSQUIC_DF_DELAY_ONCLOSE 0
+
+/**
+ * By default, maximum batch size is not specified, leaving it up to the
+ * library.
+ */
+#define LSQUIC_DF_MAX_BATCH_SIZE 0
 
 struct lsquic_engine_settings {
     /**
@@ -385,10 +483,22 @@ struct lsquic_engine_settings {
      * which means that CFCW is not allowed to increase from its initial
      * value.
      *
-     * @see es_cfcw
+     * This setting is applicable to both gQUIC and IETF QUIC.
+     *
+     * @see es_cfcw, @see es_init_max_data.
      */
     unsigned        es_max_cfcw;
 
+    /**
+     * This value is used to specify the maximum value stream flow control
+     * window is allowed to reach due to auto-tuning.  By default, this
+     * value is zero, meaning that auto-tuning is turned off.
+     *
+     * This setting is applicable to both gQUIC and IETF QUIC.
+     *
+     * @see es_sfcw, @see es_init_max_stream_data_bidi_remote,
+     * @see es_init_max_stream_data_bidi_local.
+     */
     unsigned        es_max_sfcw;
 
     /** MIDS */
@@ -408,7 +518,12 @@ struct lsquic_engine_settings {
     /** ICSL in microseconds; GQUIC only */
     unsigned long   es_idle_conn_to;
 
-    /** SCLS (silent close) */
+    /**
+     * When true, CONNECTION_CLOSE is not sent when connection times out.
+     * The server will also not send a reply to client's CONNECTION_CLOSE.
+     *
+     * Corresponds to SCLS (silent close) gQUIC option.
+     */
     int             es_silent_close;
 
     /**
@@ -458,9 +573,11 @@ struct lsquic_engine_settings {
      * (source-addr, dest-addr) tuple, thereby making it necessary to create
      * a socket for each connection.
      *
-     * This option has no effect in Q046, as the server never includes
+     * This option has no effect in Q046 and Q050, as the server never includes
      * CIDs in the short packets.
      *
+     * This setting is applicable to gQUIC only.
+     *
      * The default is @ref LSQUIC_DF_SUPPORT_TCID0.
      */
     int             es_support_tcid0;
@@ -474,6 +591,8 @@ struct lsquic_engine_settings {
      *
      * This option does not affect the server, as it must support NSTP mode
      * if it was specified by the client.
+     *
+     * This setting is applicable to gQUIC only.
      */
     int             es_support_nstp;
 
@@ -481,6 +600,8 @@ struct lsquic_engine_settings {
      * If set to true value, the library will drop connections when it
      * receives corresponding Public Reset packet.  The default is to
      * ignore these packets.
+     *
+     * The default is @ref LSQUIC_DF_HONOR_PRST.
      */
     int             es_honor_prst;
 
@@ -516,12 +637,14 @@ struct lsquic_engine_settings {
      * @ref lsquic_stream_wantread() or @ref lsquic_stream_wantwrite()
      * or shuts down the stream.
      *
+     * This also applies to the on_dg_write() callback.
+     *
      * The default value is @ref LSQUIC_DF_RW_ONCE.
      */
     int             es_rw_once;
 
     /**
-     * If set, this value specifies that number of microseconds that
+     * If set, this value specifies the number of microseconds that
      * @ref lsquic_engine_process_conns() and
      * @ref lsquic_engine_send_unsent_packets() are allowed to spend
      * before returning.
@@ -551,6 +674,42 @@ struct lsquic_engine_settings {
      */
     unsigned        es_clock_granularity;
 
+    /**
+     * Congestion control algorithm to use.
+     *
+     *  0:  Use default (@ref LSQUIC_DF_CC_ALGO)
+     *  1:  Cubic
+     *  2:  BBRv1
+     *  3:  Adaptive (Cubic or BBRv1)
+     */
+    unsigned        es_cc_algo;
+
+    /**
+     * Congestion controller RTT threshold in microseconds.
+     *
+     * Adaptive congestion control uses BBRv1 until RTT is determined.  At
+     * that point a permanent choice of congestion controller is made. If
+     * RTT is smaller than or equal to es_cc_rtt_thresh, congestion
+     * controller is switched to Cubic; otherwise, BBRv1 is picked.
+     *
+     * The default value is @ref LSQUIC_DF_CC_RTT_THRESH.
+     */
+    unsigned        es_cc_rtt_thresh;
+
+    /**
+     * No progress timeout.
+     *
+     * If connection does not make progress for this number of seconds, the
+     * connection is dropped.  Here, progress is defined as user streams
+     * being written to or read from.
+     *
+     * If this value is zero, this timeout is disabled.
+     *
+     * Default value is @ref LSQUIC_DF_NOPROGRESS_TIMEOUT_SERVER in server
+     * mode and @ref LSQUIC_DF_NOPROGRESS_TIMEOUT_CLIENT in client mode.
+     */
+    unsigned        es_noprogress_timeout;
+
     /* The following settings are specific to IETF QUIC. */
     /* vvvvvvvvvvv */
 
@@ -566,15 +725,27 @@ struct lsquic_engine_settings {
     unsigned        es_init_max_data;
 
     /**
-     * Initial max stream data.
+     * Initial maximum amount of stream data allowed to be sent on streams
+     * created by remote end (peer).
      *
      * This is a transport parameter.
      *
      * Depending on the engine mode, the default value is either
-     * @ref LSQUIC_DF_INIT_MAX_STREAM_DATA_CLIENT or
+     * @ref LSQUIC_DF_INIT_MAX_STREAM_DATA_BIDI_REMOTE_CLIENT or
      * @ref LSQUIC_DF_INIT_MAX_STREAM_DATA_BIDI_REMOTE_SERVER.
      */
     unsigned        es_init_max_stream_data_bidi_remote;
+
+    /**
+     * Initial maximum amount of stream data allowed to be sent on streams
+     * created by remote end (peer).
+     *
+     * This is a transport parameter.
+     *
+     * Depending on the engine mode, the default value is either
+     * @ref LSQUIC_DF_INIT_MAX_STREAM_DATA_BIDI_LOCAL_CLIENT or
+     * @ref LSQUIC_DF_INIT_MAX_STREAM_DATA_BIDI_LOCAL_SERVER.
+     */
     unsigned        es_init_max_stream_data_bidi_local;
 
     /**
@@ -698,15 +869,6 @@ struct lsquic_engine_settings {
      */
     int             es_allow_migration;
 
-    /**
-     * Congestion control algorithm to use.
-     *
-     *  0:  Use default (@ref LSQUIC_DF_CC_ALGO)
-     *  1:  Cubic
-     *  2:  BBR
-     */
-    unsigned        es_cc_algo;
-
     /**
      * Use QL loss bits.  Allowed values are:
      *  0:  Do not use loss bits
@@ -727,9 +889,6 @@ struct lsquic_engine_settings {
     /**
      * Enable delayed ACKs extension.  Allowed values are 0 and 1.
      *
-     * Warning: this is an experimental feature.  Using it will most likely
-     * lead to degraded performance.
-     *
      * Default value is @ref LSQUIC_DF_DELAYED_ACKS
      */
     int             es_delayed_acks;
@@ -748,9 +907,152 @@ struct lsquic_engine_settings {
      *
      * If set to zero, limit is not set.
      *
-     * Default value is @ref LSQUIC_DF_MAX_PACKET_SIZE_RX
+     * Default value is @ref LSQUIC_DF_MAX_UDP_PAYLOAD_SIZE_RX
      */
-    unsigned short  es_max_packet_size_rx;
+    unsigned short  es_max_udp_payload_size_rx;
+
+    /**
+     * Enable the "QUIC bit grease" extension.  When set to a true value,
+     * lsquic will grease the QUIC bit on the outgoing QUIC packets if
+     * the peer sent the "grease_quic_bit" transport parameter.
+     *
+     * Default value is @ref LSQUIC_DF_GREASE_QUIC_BIT
+     */
+    int             es_grease_quic_bit;
+
+    /**
+     * If set to true value, enable DPLPMTUD -- Datagram Packetization
+     * Layer Path MTU Discovery.
+     *
+     * Default value is @ref LSQUIC_DF_DPLPMTUD
+     */
+    int             es_dplpmtud;
+
+    /**
+     * PLPMTU size expected to work for most paths.
+     *
+     * If set to zero, this value is calculated based on QUIC and IP versions.
+     *
+     * Default value is @ref LSQUIC_DF_BASE_PLPMTU.
+     */
+    unsigned short  es_base_plpmtu;
+
+    /**
+     * Largest PLPMTU size the engine will try.
+     *
+     * If set to zero, picking this value is left to the engine.
+     *
+     * Default value is @ref LSQUIC_DF_MAX_PLPMTU.
+     */
+    unsigned short  es_max_plpmtu;
+
+    /**
+     * This value specifies how long the DPLPMTUD probe timer is, in
+     * milliseconds.  [draft-ietf-tsvwg-datagram-plpmtud-17] says:
+     *
+     " PROBE_TIMER:  The PROBE_TIMER is configured to expire after a period
+     "    longer than the maximum time to receive an acknowledgment to a
+     "    probe packet.  This value MUST NOT be smaller than 1 second, and
+     "    SHOULD be larger than 15 seconds.  Guidance on selection of the
+     "    timer value are provided in section 3.1.1 of the UDP Usage
+     "    Guidelines [RFC8085].
+     *
+     * If set to zero, the default is used.
+     *
+     * Default value is @ref LSQUIC_DF_MTU_PROBE_TIMER.
+     */
+    unsigned        es_mtu_probe_timer;
+
+    /**
+     * Enable datagram extension.  Allowed values are 0 and 1.
+     *
+     * Default value is @ref LSQUIC_DF_DATAGRAMS
+     */
+    int             es_datagrams;
+
+    /**
+     * If set to true, changes in peer port are assumed to be due to a
+     * benign NAT rebinding and path characteristics -- MTU, RTT, and
+     * CC state -- are not reset.
+     *
+     * Default value is @ref LSQUIC_DF_OPTIMISTIC_NAT.
+     */
+    int             es_optimistic_nat;
+
+    /**
+     * If set to true, Extensible HTTP Priorities are enabled.  This
+     * is HTTP/3-only setting.
+     *
+     * Default value is @ref LSQUIC_DF_EXT_HTTP_PRIO
+     */
+    int             es_ext_http_prio;
+
+    /**
+     * If set to 1, QPACK statistics are logged per connection.
+     *
+     * If set to 2, QPACK experiments are run.  In this mode, encoder
+     * and decoder setting values are randomly selected (from the range
+     * [0, whatever is specified in es_qpack_(enc|dec)_*]) and these
+     * values along with compression ratio and user agent are logged at
+     * NOTICE level when connection is destroyed.  The purpose of these
+     * experiments is to use compression performance statistics to figure
+     * out a good set of default values.
+     *
+     * Default value is @ref LSQUIC_DF_QPACK_EXPERIMENT.
+     */
+    int             es_qpack_experiment;
+
+    /**
+     * Settings for the Packet Tolerance PID Controller (PTPC) used for
+     * the Delayed ACKs logic.  Periodicity is how often the number of
+     * incoming ACKs is sampled.  Periodicity's units is the number of
+     * RTTs. Target is the average number of incoming ACKs per RTT we
+     * want to achieve.  Error threshold defines the range of error values
+     * within which no action is taken.  For example, error threshold of
+     * 0.03 means that adjustment actions will be taken only when the
+     * error is outside of the [-0.03, 0.03] range.  Proportional and
+     * integral gains have their usual meanings described here:
+     *      https://en.wikipedia.org/wiki/PID_controller#Controller_theory
+     *
+     * The average is normalized as follows:
+     *    AvgNormalized = Avg * e / Target      # Where 'e' is 2.71828...
+     *
+     * The error is then calculated as ln(AvgNormalized) - 1.  This gives
+     * us a logarithmic scale that is convenient to use for adjustment
+     * calculations.  The error divisor is used to calculate the packet
+     * tolerance adjustment:
+     *    Adjustment = Error / ErrorDivisor
+     *
+     * WARNING.  The library comes with sane defaults.  Only fiddle with
+     * these knobs if you know what you are doing.
+     */
+    unsigned es_ptpc_periodicity;   /* LSQUIC_DF_PTPC_PERIODICITY */
+    unsigned es_ptpc_max_packtol;   /* LSQUIC_DF_PTPC_MAX_PACKTOL */
+    int      es_ptpc_dyn_target;    /* LSQUIC_DF_PTPC_DYN_TARGET */
+    float    es_ptpc_target,        /* LSQUIC_DF_PTPC_TARGET */
+             es_ptpc_prop_gain,     /* LSQUIC_DF_PTPC_PROP_GAIN */
+             es_ptpc_int_gain,      /* LSQUIC_DF_PTPC_INT_GAIN */
+             es_ptpc_err_thresh,    /* LSQUIC_DF_PTPC_ERR_THRESH */
+             es_ptpc_err_divisor;   /* LSQUIC_DF_PTPC_ERR_DIVISOR */
+
+    /**
+     * When set to true, the on_close() callback will be delayed until the
+     * peer acknowledges all data sent on the stream.  (Or until the connection
+     * is destroyed in some manner -- either explicitly closed by the user or
+     * as a result of an engine shutdown.)
+     *
+     * Default value is @ref LSQUIC_DF_DELAY_ONCLOSE
+     */
+    int             es_delay_onclose;
+
+    /**
+     * If set to a non-zero value, specifies maximum batch size.  (The
+     * batch of packets passed to @ref ea_packets_out() callback).  Must
+     * be no larger than 1024.
+     *
+     * Default value is @ref LSQUIC_DF_MAX_BATCH_SIZE
+     */
+    unsigned        es_max_batch_size;
 };
 
 /* Initialize `settings' to default values */
@@ -786,7 +1088,8 @@ struct lsquic_out_spec
     const struct sockaddr *local_sa;
     const struct sockaddr *dest_sa;
     void                  *peer_ctx;
-    int                    ecn; /* Valid values are 0 - 3.  See RFC 3168 */
+    lsquic_conn_ctx_t     *conn_ctx;  /* will be NULL when sending out the first batch of handshake packets */
+    int                    ecn;       /* Valid values are 0 - 3.  See RFC 3168 */
 };
 
 /**
@@ -855,19 +1158,19 @@ struct lsquic_packout_mem_if
     /**
      * Allocate buffer for sending.
      */
-    void *  (*pmi_allocate) (void *pmi_ctx, void *conn_ctx, unsigned short sz,
+    void *  (*pmi_allocate) (void *pmi_ctx, void *peer_ctx, lsquic_conn_ctx_t *, unsigned short sz,
                                                                 char is_ipv6);
     /**
      * This function is used to release the allocated buffer after it is
      * sent via @ref ea_packets_out.
      */
-    void    (*pmi_release)  (void *pmi_ctx, void *conn_ctx, void *buf,
+    void    (*pmi_release)  (void *pmi_ctx, void *peer_ctx, void *buf,
                                                                 char is_ipv6);
     /**
      * If allocated buffer is not going to be sent, return it to the caller
      * using this function.
      */
-    void    (*pmi_return)  (void *pmi_ctx, void *conn_ctx, void *buf,
+    void    (*pmi_return)  (void *pmi_ctx, void *peer_ctx, void *buf,
                                                                 char is_ipv6);
 };
 
@@ -876,37 +1179,17 @@ typedef void (*lsquic_cids_update_f)(void *ctx, void **peer_ctx,
 
 struct stack_st_X509;
 
-/**
- * When headers are processed, various errors may occur.  They are listed
- * in this enum.
- */
-enum lsquic_header_status
-{
-    LSQUIC_HDR_OK,
-    /** Duplicate pseudo-header */
-    LSQUIC_HDR_ERR_DUPLICATE_PSDO_HDR,
-    /** Not all request pseudo-headers are present */
-    LSQUIC_HDR_ERR_INCOMPL_REQ_PSDO_HDR,
-    /** Unnecessary request pseudo-header present in the response */
-    LSQUIC_HDR_ERR_UNNEC_REQ_PSDO_HDR,
-    /** Prohibited header in request */
-    LSQUIC_HDR_ERR_BAD_REQ_HEADER,
-    /** Not all response pseudo-headers are present */
-    LSQUIC_HDR_ERR_INCOMPL_RESP_PSDO_HDR,
-    /** Unnecessary response pseudo-header present in the response. */
-    LSQUIC_HDR_ERR_UNNEC_RESP_PSDO_HDR,
-    /** Unknown pseudo-header */
-    LSQUIC_HDR_ERR_UNKNOWN_PSDO_HDR,
-    /** Uppercase letter in header */
-    LSQUIC_HDR_ERR_UPPERCASE_HEADER,
-    /** Misplaced pseudo-header */
-    LSQUIC_HDR_ERR_MISPLACED_PSDO_HDR,
-    /** Missing pseudo-header */
-    LSQUIC_HDR_ERR_MISSING_PSDO_HDR,
-    /** Header or headers are too large */
-    LSQUIC_HDR_ERR_HEADERS_TOO_LARGE,
-    /** Cannot allocate any more memory. */
-    LSQUIC_HDR_ERR_NOMEM,
+enum lsquic_hsi_flag {
+    /**
+     * Turn HTTP/1.x mode on or off.  In this mode, decoded name and value
+     * pair are separated by ": " and "\r\n" is appended to the end of the
+     * string.  By default, this mode is off.
+     */
+    LSQUIC_HSI_HTTP1X          = 1 << 1,
+    /** Include name hash into lsxpack_header */
+    LSQUIC_HSI_HASH_NAME       = 1 << 2,
+    /** Include nameval hash into lsxpack_header */
+    LSQUIC_HSI_HASH_NAMEVAL    = 1 << 3,
 };
 
 struct lsquic_hset_if
@@ -915,56 +1198,58 @@ struct lsquic_hset_if
      * Create a new header set.  This object is (and must be) fetched from a
      * stream by calling @ref lsquic_stream_get_hset() before the stream can
      * be read.
+     *
+     * `stream' may be set to NULL in server mode.
      */
-    void *              (*hsi_create_header_set)(void *hsi_ctx,
-                                                        int is_push_promise);
+    void * (*hsi_create_header_set)(void *hsi_ctx, lsquic_stream_t *stream,
+                                    int is_push_promise);
     /**
-     * Process new header.  Return 0 on success, -1 if there is a problem with
-     * the header.  -1 is treated as a stream error: the associated stream is
-     * reset.
+     * Return a header set prepared for decoding.  If `hdr' is NULL, this
+     * means return a new structure with at least `space' bytes available
+     * in the decoder buffer.  On success, a newly prepared header is
+     * returned.
+     *
+     * If `hdr' is not NULL, it means there was not enough decoder buffer
+     * and it must be increased to at least `space' bytes.  `buf', `val_len',
+     * and `name_offset' member of the `hdr' structure may change.  On
+     * success, the return value is the same as `hdr'.
+     *
+     * If NULL is returned, the space cannot be allocated.
+     */
+    struct lsxpack_header *
+                        (*hsi_prepare_decode)(void *hdr_set,
+                                              struct lsxpack_header *hdr,
+                                              size_t space);
+    /**
+     * Process new header.  Return 0 on success, a positive value if a header
+     * error occured, or a negative value on any other error.
+     *
+     * A positive return value will result in cancellation of associated
+     * stream.
+     *
+     * A negative return value will result in connection being aborted.
      *
      * `hdr_set' is the header set object returned by
      * @ref hsi_create_header_set().
      *
-     * `name_idx' is set to the index in either the HPACK or QPACK static table
-     * whose entry's name element matches `name'.  The values are as follows:
-     *      - if there is no such match, `name_idx' is set to zero;
-     *      - if HPACK is used, the value is between 1 and 61; and
-     *      - if QPACK is used, the value is 62+ (subtract 62 to get the QPACK
-     *        static table index).
+     * `hdr' is the header returned by @ref `hsi_prepare_decode'.
      *
-     * If `name' is NULL, this means that no more header are going to be
+     * If `hdr' is NULL, this means that no more header are going to be
      * added to the set.
      */
-    enum lsquic_header_status (*hsi_process_header)(void *hdr_set,
-                                    unsigned name_idx,
-                                    const char *name, unsigned name_len,
-                                    const char *value, unsigned value_len);
+    int (*hsi_process_header)(void *hdr_set, struct lsxpack_header *hdr);
     /**
      * Discard header set.  This is called for unclaimed header sets and
      * header sets that had an error.
      */
     void                (*hsi_discard_header_set)(void *hdr_set);
-};
-
-/**
- * SSL keylog interface.
- */
-struct lsquic_keylog_if
-{
-    /** Return keylog handle or NULL if no key logging is desired */
-    void *    (*kli_open) (void *keylog_ctx, lsquic_conn_t *);
-
     /**
-     * Log line.  The first argument is the pointer returned by
-     * @ref kli_open.
+     * These flags specify properties of decoded headers passed to
+     * hsi_process_header().  This is only applicable to QPACK headers;
+     * HPACK library header properties are based on compilation, not
+     * run-time, options.
      */
-    void      (*kli_log_line) (void *handle, const char *line);
-
-    /**
-     * Close handle.
-     */
-    void      (*kli_close) (void *handle);
+    enum lsquic_hsi_flag hsi_flags;
 };
 
 /**
@@ -992,7 +1277,9 @@ struct lsquic_engine_api
     /** Function to look up certificates by SNI is used in server mode. */
     lsquic_lookup_cert_f                 ea_lookup_cert;
     void                                *ea_cert_lu_ctx;
-    struct ssl_ctx_st *                (*ea_get_ssl_ctx)(void *peer_ctx);
+    /** Mandatory callback for server, optional for client. */
+    struct ssl_ctx_st *                (*ea_get_ssl_ctx)(void *peer_ctx,
+                                                const struct sockaddr *local);
     /**
      * Shared hash interface is optional.  If set to zero, performance of
      * multiple LSQUIC instances will be degraded.
@@ -1033,25 +1320,25 @@ struct lsquic_engine_api
      */
     const struct lsquic_hset_if         *ea_hsi_if;
     void                                *ea_hsi_ctx;
-#if LSQUIC_CONN_STATS
+
     /**
      * If set, engine will print cumulative connection statistics to this
-     * file just before it is destroyed.
+     * file just before it is destroyed.  (Must be compiled with
+     * -DLSQUIC_CONN_STATS=1).
      */
     void /* FILE, really */             *ea_stats_fh;
-#endif
 
     /**
-     * Optional SSL key logging interface.
-     */
-    const struct lsquic_keylog_if       *ea_keylog_if;
-    void                                *ea_keylog_ctx;
-
-    /**
-     * The optional ALPN string is used by the client @ref LSENG_HTTP
+     * The optional ALPN string is used by the client if @ref LSENG_HTTP
      * is not set.
      */
     const char                          *ea_alpn;
+
+    /**
+     * Optional interface to control the creation of connection IDs
+     */
+    void                               (*ea_generate_scid)(lsquic_conn_t *,
+                                                    lsquic_cid_t *, unsigned);
 };
 
 /**
@@ -1074,19 +1361,19 @@ lsquic_engine_new (unsigned lsquic_engine_flags,
 /**
  * Create a client connection to peer identified by `peer_ctx'.
  *
- * To let the engine specify QUIC version, use N_LSQVER.  If zero-rtt info
- * is supplied, version is picked from there instead.
+ * To let the engine specify QUIC version, use N_LSQVER.  If session resumption
+ * information is supplied, version is picked from there instead.
  *
- * If `max_packet_size' is set to zero, it is inferred based on `peer_sa':
- * 1350 for IPv6 and 1370 for IPv4.
+ * If `base_plpmtu' is set to zero, it is selected based on the
+ * engine settings, QUIC version, and IP version.
  */
 lsquic_conn_t *
 lsquic_engine_connect (lsquic_engine_t *, enum lsquic_version,
                        const struct sockaddr *local_sa,
                        const struct sockaddr *peer_sa,
                        void *peer_ctx, lsquic_conn_ctx_t *conn_ctx,
-                       const char *hostname, unsigned short max_packet_size,
-                       const unsigned char *zero_rtt, size_t zero_rtt_len,
+                       const char *hostname, unsigned short base_plpmtu,
+                       const unsigned char *sess_resume, size_t sess_resume_len,
                        /** Resumption token: optional */
                        const unsigned char *token, size_t token_sz);
 
@@ -1119,7 +1406,8 @@ lsquic_engine_process_conns (lsquic_engine_t *engine);
 
 /**
  * Returns true if engine has some unsent packets.  This happens if
- * @ref ea_packets_out() could not send everything out.
+ * @ref ea_packets_out() could not send everything out or if processing
+ * deadline was exceeded (see @ref es_proc_time_thresh).
  */
 int
 lsquic_engine_has_unsent_packets (lsquic_engine_t *engine);
@@ -1173,12 +1461,7 @@ lsquic_conn_cancel_pending_streams (lsquic_conn_t *, unsigned n);
  * Mark connection as going away: send GOAWAY frame and do not accept
  * any more incoming streams, nor generate streams of our own.
  *
- * In the server mode, of course, we can call this function just fine in both
- * Google and IETF QUIC.
- *
- * In client mode, calling this function in for an IETF QUIC connection does
- * not do anything, as the client MUST NOT send GOAWAY frames.
- * See [draft-ietf-quic-http-17] Section 4.2.7.
+ * Only applicable to HTTP/3 and GQUIC connections.  Otherwise a no-op.
  */
 void
 lsquic_conn_going_away (lsquic_conn_t *);
@@ -1267,6 +1550,14 @@ lsquic_stream_write (lsquic_stream_t *s, const void *buf, size_t len);
 ssize_t
 lsquic_stream_writev (lsquic_stream_t *s, const struct iovec *vec, int count);
 
+/**
+ * Write to streams using a single call to a preadv-like function.
+ */
+ssize_t
+lsquic_stream_pwritev (lsquic_stream_t *s,
+    ssize_t (*preadv)(void *user_data, const struct iovec *iov, int iovcnt),
+    void *user_data, size_t n_to_write);
+
 /**
  * Used as argument to @ref lsquic_stream_writef()
  */
@@ -1306,17 +1597,6 @@ lsquic_stream_writef (lsquic_stream_t *, struct lsquic_reader *);
 int
 lsquic_stream_flush (lsquic_stream_t *s);
 
-/**
- * @typedef lsquic_http_header_t
- * @brief HTTP header structure. Contains header name and value.
- *
- */
-typedef struct lsquic_http_header
-{
-   struct iovec name;
-   struct iovec value;
-} lsquic_http_header_t;
-
 /**
  * @typedef lsquic_http_headers_t
  * @brief HTTP header list structure. Contains a list of HTTP headers in key/value pairs.
@@ -1325,7 +1605,7 @@ typedef struct lsquic_http_header
 struct lsquic_http_headers
 {
     int                     count;
-    lsquic_http_header_t   *headers;
+    struct lsxpack_header  *headers;
 };
 
 /**
@@ -1356,10 +1636,7 @@ lsquic_stream_get_hset (lsquic_stream_t *);
  * trigger on_new_stream() event and it can be used as a regular client-
  * initiated stream.
  *
- * If `hdr_set' is not set, it is generated by using `ea_hsi_if' callbacks.
- * In either case, the header set object belongs to the connection.  The
- * user is not to free this object until (@ref hsi_discard_header_set) is
- * called.
+ * `hdr_set' must be set.  It is passed as-is to @lsquic_stream_get_hset.
  *
  * @retval  0   Stream pushed successfully.
  * @retval  1   Stream push failed because it is disabled or because we hit
@@ -1368,7 +1645,6 @@ lsquic_stream_get_hset (lsquic_stream_t *);
  */
 int
 lsquic_conn_push_stream (lsquic_conn_t *c, void *hdr_set, lsquic_stream_t *s,
-    const  struct iovec* url, const struct iovec* authority,
     const lsquic_http_headers_t *headers);
 
 /**
@@ -1383,6 +1659,13 @@ int lsquic_stream_shutdown(lsquic_stream_t *s, int how);
 
 int lsquic_stream_close(lsquic_stream_t *s);
 
+/**
+ * Return true if peer has not ACKed all data written to the stream.  This
+ * includes both packetized and buffered data.
+ */
+int
+lsquic_stream_has_unacked_data (lsquic_stream_t *s);
+
 /**
  * Get certificate chain returned by the server.  This can be used for
  * server certificate verification.
@@ -1444,12 +1727,49 @@ unsigned lsquic_stream_priority (const lsquic_stream_t *s);
 
 /**
  * Set stream priority.  Valid priority values are 1 through 256, inclusive.
+ * Lower value means higher priority.
  *
  * @retval   0  Success.
  * @retval  -1  Priority value is invalid.
  */
 int lsquic_stream_set_priority (lsquic_stream_t *s, unsigned priority);
 
+/*
+ * Definitions for Extensible HTTP Priorities:
+ * https://tools.ietf.org/html/draft-ietf-httpbis-priority-01
+ */
+/* This is maximum *value* -- but it's the lowest *priority* */
+#define LSQUIC_MAX_HTTP_URGENCY 7
+#define LSQUIC_DEF_HTTP_URGENCY 3
+#define LSQUIC_DEF_HTTP_INCREMENTAL 0
+
+struct lsquic_ext_http_prio
+{
+    unsigned char   urgency;
+    signed char     incremental;
+};
+
+/**
+ * Get Extensible HTTP Priorities associated with the stream.
+ *
+ * Returns zero on success of a negative value on failure.  A failure occurs
+ * if this is not an HTTP/3 stream or if Extensible HTTP Priorities haven't
+ * been enabled.  See @ref es_ext_http_prio.
+ */
+int
+lsquic_stream_get_http_prio (lsquic_stream_t *, struct lsquic_ext_http_prio *);
+
+/**
+ * Set Extensible HTTP Priorities of the stream.
+ *
+ * Returns zero on success of a negative value on failure.  A failure occurs
+ * if some internal error occured or if this is not an HTTP/3 stream or if
+ * Extensible HTTP Priorities haven't been enabled.  See @ref es_ext_http_prio.
+ */
+int
+lsquic_stream_set_http_prio (lsquic_stream_t *,
+                                        const struct lsquic_ext_http_prio *);
+
 /**
  * Get a pointer to the connection object.  Use it with lsquic_conn_*
  * functions.
@@ -1468,6 +1788,20 @@ int
 lsquic_conn_get_sockaddr(lsquic_conn_t *c,
                 const struct sockaddr **local, const struct sockaddr **peer);
 
+/* Returns previous value */
+int
+lsquic_conn_want_datagram_write (lsquic_conn_t *, int is_want);
+
+/* Get minimum datagram size.  By default, this value is zero. */
+size_t
+lsquic_conn_get_min_datagram_size (lsquic_conn_t *);
+
+/* Set minimum datagram size.  This is the minumum value of the buffer passed
+ * to the on_dg_write() callback.
+ */
+int
+lsquic_conn_set_min_datagram_size (lsquic_conn_t *, size_t sz);
+
 struct lsquic_logger_if {
     int     (*log_buf)(void *logger_ctx, const char *buf, size_t len);
 };
@@ -1645,6 +1979,10 @@ lsquic_conn_set_ctx (lsquic_conn_t *, lsquic_conn_ctx_t *);
 void *
 lsquic_conn_get_peer_ctx (lsquic_conn_t *, const struct sockaddr *local_sa);
 
+/** Get SNI sent by the client */
+const char *
+lsquic_conn_get_sni (lsquic_conn_t *);
+
 /**
  * Abort connection.
  */
@@ -1723,6 +2061,10 @@ lsquic_conn_status (lsquic_conn_t *, char *errbuf, size_t bufsz);
 extern const char *const
 lsquic_ver2str[N_LSQVER];
 
+/* Return connection associated with this SSL object */
+lsquic_conn_t *
+lsquic_ssl_to_conn (const struct ssl_st *);
+
 #ifdef __cplusplus
 }
 #endif
