@@ -195,15 +195,15 @@ void ExtAppSubRegistry::onTimer()
          iter != m_pRegistry->end();
          iter = m_pRegistry->next(iter))
         iter.second()->onTimer();
-    
+
     //Every 10 seconds to check the restart file changed or not
     struct stat st;
     if(stat(s_lsphpRestartFlagFile, &st) == 0)
     {
         LocalWorker::s_tmRestartPhp = st.st_mtime;
     }
-    
-    
+
+
 }
 
 
@@ -423,7 +423,7 @@ int ExtAppRegistry::configVhostOwnPhp(HttpVHost *pVHost)
 
         /**
          * Add the VHost name to the end of the pURI to avoid conflict with others
-         * 
+         *
          */
         if (addr.set(pUri, NO_ANY | DO_NSLOOKUP))
         {
@@ -433,9 +433,9 @@ int ExtAppRegistry::configVhostOwnPhp(HttpVHost *pVHost)
 
         iAutoStart = ConfigCtx::getCurConfigCtx()->
                                 getLongValue(pNode, "autoStart", 0, 2, 1);
-    
+
         pPath = pNode->getChildValue("path");
-        
+
         if (iAutoStart)
         {
             if (ConfigCtx::getCurConfigCtx()->getAbsoluteFile(buf, pPath) != 0)
@@ -509,7 +509,7 @@ int ExtAppRegistry::configVhostOwnPhp(HttpVHost *pVHost)
             config.setUGid(pVHost->getUid(), pVHost->getGid());
         }
     }
-    
+
     return 0;
 }
 
@@ -544,6 +544,71 @@ void ExtAppRegistry::getUniAppUri(const char *app_uri, char *dst, int dst_len, i
 }
 
 
+static bool isBlackListed(const char *key, const char **blacklist, int size)
+{
+    for (int i = 0; i < size; ++i)
+        if (0 == strcmp(blacklist[i], key))
+            return true;
+    return false;
+}
+
+
+static int validateBinPath(const char *path)
+{
+    const char *p;
+    if (!path)
+        return LS_OK;
+    if (strstr(path, "/../"))
+        return LS_FAIL;
+    p = strrchr(path, '/');
+    if (!p)
+        p = path;
+    else
+        p += 1;
+
+    static const char *s_blocked[] =
+    {   "bash", "dash", "ksh", "sh", "ncat", "nc"   };
+
+    if (isBlackListed(p, s_blocked, sizeof(s_blocked)/sizeof(const char *)))
+        return LS_FAIL;
+    return LS_OK;
+}
+
+
+static int validateCmdParam(const char *param)
+{
+    const char *p = param;
+    while(isspace(*p))
+        ++p;
+    if (!*p)
+        return LS_OK;
+    if (strstr(param, "/dev/tcp/") != NULL)
+        return LS_FAIL;
+
+    const char *s_blocked_cmd[] =
+    {   "bash", "dash", "ksh", "sh"     };
+    int s_blocked_cmd_len[] =
+    {   4, 4, 3, 2  };
+
+    for(uint i = 0; i < sizeof(s_blocked_cmd) / sizeof(const char *); ++i)
+    {
+        const char *begin = param;
+        while((p = strstr(begin, s_blocked_cmd[i])) != NULL)
+        {
+            char ch = *(p - 1);
+            if (isspace(ch) || (ispunct(ch) && ch != '.' && ch != '_' && ch != '-'))
+            {
+                char after = *(p + s_blocked_cmd_len[i]);
+                if (after == '\0' || isspace(after)
+                    || (ispunct(ch) && ch != '.' && ch != '_' && ch != '-'))
+                    return LS_FAIL;
+            }
+            begin += s_blocked_cmd_len[i];
+        }
+    }
+    return LS_OK;
+}
+
 
 /**
  * When it is from server level, the pVhost is NULL
@@ -595,19 +660,19 @@ ExtWorker *ExtAppRegistry::configExtApp(const XmlNode *pNode, const HttpVHost *p
 
 
     /**
-     * For proxy type, will not add the uid to the name to avoid 
+     * For proxy type, will not add the uid to the name to avoid
      * can not find it later
      */
-    if(pVHost && iType != HandlerType::HT_PROXY)
+    if(pVHost && iType != HandlerType::HT_PROXY && iType != HandlerType::HT_LOGGER)
     {
         pVHost->getUniAppName(pName, appNameVh, 256);
         pName = appNameVh;
     }
     ConfigCtx currentCtx(pName);
-    
+
     iType -= HandlerType::HT_CGI;
-    
-    
+
+
     pWorker = getApp(iType, pName);
     if (pWorker)
     {
@@ -615,7 +680,7 @@ ExtWorker *ExtAppRegistry::configExtApp(const XmlNode *pNode, const HttpVHost *p
                 pName);
         return pWorker;
     }
-    
+
 
     pUri = ConfigCtx::getCurConfigCtx()->getExpandedTag(pNode, "address",
             achAddress, 128);
@@ -672,7 +737,7 @@ ExtWorker *ExtAppRegistry::configExtApp(const XmlNode *pNode, const HttpVHost *p
                         pUri, appUriVh);
                 pUri = appUriVh;
             }
-            
+
             if (uriInUse)
             {
                 LS_ERROR(&currentCtx, "socket address %s is used, give up!", pUri);
@@ -692,7 +757,7 @@ ExtWorker *ExtAppRegistry::configExtApp(const XmlNode *pNode, const HttpVHost *p
                 }
             }
         }
-            
+
         pPath = pNode->getChildValue("path");
 
         if ((iAutoStart) && ((!pPath || !*pPath)))
@@ -712,6 +777,14 @@ ExtWorker *ExtAppRegistry::configExtApp(const XmlNode *pNode, const HttpVHost *p
             if (p)
                 *p = 0;
 
+            if (validateBinPath(pCmd) == LS_FAIL)
+            {
+                LS_ERROR(&currentCtx, "invalid command path - '%s', "
+                         "it cannot be started by Web server!",
+                         pCmd);
+                return NULL;
+            }
+
             if (access(pCmd, X_OK) == -1)
             {
                 LS_ERROR(&currentCtx, "invalid path - %s, "
@@ -720,7 +793,15 @@ ExtWorker *ExtAppRegistry::configExtApp(const XmlNode *pNode, const HttpVHost *p
             }
 
             if (p)
+            {
                 *p = ' ';
+                if (validateCmdParam(p + 1) == LS_FAIL)
+                {
+                    LS_ERROR(&currentCtx, "invalid command parameter - '%s', "
+                            "it cannot be started by Web server!", p + 1);
+                    return NULL;
+                }
+            }
 
             if (pCmd != buf)
                 buf[len] = *buf;
@@ -735,7 +816,7 @@ ExtWorker *ExtAppRegistry::configExtApp(const XmlNode *pNode, const HttpVHost *p
         LS_ERROR(&currentCtx, "failed to add external processor: %s!", pName);
         return NULL;
     }
-    
+
     pConfig = pWorker->getConfigPointer();
     assert(pConfig);
 
@@ -790,10 +871,10 @@ ExtWorker *ExtAppRegistry::configExtApp(const XmlNode *pNode, const HttpVHost *p
         }
     }
 
-        
+
     /***
      * If is server level, need to save the XmlNode
-     * 
+     *
      */
     if (!pVHost)
     {
@@ -809,8 +890,8 @@ ExtWorker *ExtAppRegistry::configExtApp(const XmlNode *pNode, const HttpVHost *p
                       pName, pNode, MAX_EXT_APP_NUMBER);
         }
     }
-    
-    
+
+
     return pWorker;
 
 }
