@@ -1,4 +1,4 @@
-/* Copyright (c) 2017 - 2019 LiteSpeed Technologies Inc.  See LICENSE. */
+/* Copyright (c) 2017 - 2021 LiteSpeed Technologies Inc.  See LICENSE. */
 /*
  * lsquic_alarmset.c -- A set of alarms
  */
@@ -32,7 +32,7 @@ lsquic_alarmset_init_alarm (lsquic_alarmset_t *alset, enum alarm_id al_id,
 }
 
 
-static const char *const lsquic_alid2str[] =
+const char *const lsquic_alid2str[] =
 {
     [AL_HANDSHAKE]  =  "HANDSHAKE",
     [AL_RETX_INIT]  =  "RETX_INIT",
@@ -40,14 +40,15 @@ static const char *const lsquic_alid2str[] =
     [AL_RETX_APP]   =  "RETX_APP",
     [AL_PING]       =  "PING",
     [AL_IDLE]       =  "IDLE",
-    [AL_ACK_INIT]   =  "ACK_INIT",
-    [AL_ACK_HSK]    =  "ACK_HSK",
     [AL_ACK_APP]    =  "ACK_APP",
     [AL_RET_CIDS]   =  "RET_CIDS",
     [AL_CID_THROT]  =  "CID_THROT",
     [AL_PATH_CHAL_0] = "PATH_CHAL_0",
     [AL_PATH_CHAL_1] = "PATH_CHAL_1",
     [AL_SESS_TICKET] = "SESS_TICKET",
+    [AL_BLOCKED_KA] = "BLOCKED_KA",
+    [AL_MTU_PROBE]  = "MTU_PROBE",
+    [AL_PACK_TOL]   = "PACK_TOL",
 };
 
 
@@ -75,20 +76,22 @@ lsquic_alarmset_ring_expired (lsquic_alarmset_t *alset, lsquic_time_t now)
 
 
 lsquic_time_t
-lsquic_alarmset_mintime (const lsquic_alarmset_t *alset)
+lsquic_alarmset_mintime (const lsquic_alarmset_t *alset, enum alarm_id *idp)
 {
     lsquic_time_t expiry;
-    enum alarm_id al_id;
+    enum alarm_id al_id, ret_id;
 
     if (alset->as_armed_set)
     {
         expiry = UINT64_MAX;
-        for (al_id = 0; al_id < MAX_LSQUIC_ALARMS; ++al_id)
+        for (al_id = 0, ret_id = 0; al_id < MAX_LSQUIC_ALARMS; ++al_id)
             if ((alset->as_armed_set & (1 << al_id))
                                 && alset->as_expiry[al_id] < expiry)
             {
                 expiry = alset->as_expiry[al_id];
+                ret_id = al_id;
             }
+        *idp = ret_id;
         return expiry;
     }
     else
