@@ -1,4 +1,4 @@
-/* Copyright (c) 2017 - 2020 LiteSpeed Technologies Inc.  See LICENSE. */
+/* Copyright (c) 2017 - 2021 LiteSpeed Technologies Inc.  See LICENSE. */
 #include <assert.h>
 #include <inttypes.h>
 #include <string.h>
@@ -65,10 +65,10 @@ int
 lsquic_conn_copy_and_release_pi_data (const lsquic_conn_t *conn,
           struct lsquic_engine_public *enpub, lsquic_packet_in_t *packet_in)
 {
+    unsigned char *copy;
+
     assert(!(packet_in->pi_flags & PI_OWN_DATA));
-    /* The size should be guarded in lsquic_engine_packet_in(): */
-    assert(packet_in->pi_data_sz <= GQUIC_MAX_PACKET_SZ);
-    unsigned char *const copy = lsquic_mm_get_packet_in_buf(&enpub->enp_mm, 1370);
+    copy = lsquic_mm_get_packet_in_buf(&enpub->enp_mm, packet_in->pi_data_sz);
     if (!copy)
     {
         LSQ_WARN("cannot allocate memory to copy incoming packet data");
@@ -196,25 +196,23 @@ lsquic_conn_get_engine (struct lsquic_conn *lconn)
 
 int
 lsquic_conn_push_stream (struct lsquic_conn *lconn, void *hset,
-    struct lsquic_stream *stream, const struct iovec* url,
-    const struct iovec *authority, const struct lsquic_http_headers *headers)
+    struct lsquic_stream *stream, const struct lsquic_http_headers *headers)
 {
-    return lconn->cn_if->ci_push_stream(lconn, hset, stream, url, authority,
-                                        headers);
+    return lconn->cn_if->ci_push_stream(lconn, hset, stream, headers);
 }
 
 
 lsquic_conn_ctx_t *
 lsquic_conn_get_ctx (const struct lsquic_conn *lconn)
 {
-    return lconn->cn_if->ci_get_ctx(lconn);
+    return lconn->cn_conn_ctx;
 }
 
 
 void
 lsquic_conn_set_ctx (struct lsquic_conn *lconn, lsquic_conn_ctx_t *ctx)
 {
-    lconn->cn_if->ci_set_ctx(lconn, ctx);
+    lconn->cn_conn_ctx = ctx;
 }
 
 
@@ -240,6 +238,14 @@ lsquic_generate_cid (lsquic_cid_t *cid, size_t len)
 }
 
 
+void
+lsquic_generate_scid (struct lsquic_conn *lconn, lsquic_cid_t *scid,
+                                                                unsigned len)
+{
+    lsquic_generate_cid(scid, len);
+}
+
+
 void
 lsquic_generate_cid_gquic (lsquic_cid_t *cid)
 {
@@ -269,3 +275,62 @@ lsquic_conn_log_cid (const struct lsquic_conn *lconn)
         return lconn->cn_if->ci_get_log_cid(lconn);
     return CN_SCID(lconn);
 }
+
+
+int
+lsquic_conn_want_datagram_write (struct lsquic_conn *lconn, int is_want)
+{
+    if (lconn->cn_if && lconn->cn_if->ci_want_datagram_write)
+        return lconn->cn_if->ci_want_datagram_write(lconn, is_want);
+    else
+        return -1;
+}
+
+
+int
+lsquic_conn_set_min_datagram_size (struct lsquic_conn *lconn, size_t sz)
+{
+    if (lconn->cn_if && lconn->cn_if->ci_set_min_datagram_size)
+        return lconn->cn_if->ci_set_min_datagram_size(lconn, sz);
+    else
+        return -1;
+}
+
+
+size_t
+lsquic_conn_get_min_datagram_size (struct lsquic_conn *lconn)
+{
+    if (lconn->cn_if && lconn->cn_if->ci_get_min_datagram_size)
+        return lconn->cn_if->ci_get_min_datagram_size(lconn);
+    else
+        return 0;
+}
+
+
+#if LSQUIC_CONN_STATS
+void
+lsquic_conn_stats_diff (const struct conn_stats *cumulative_stats,
+                        const struct conn_stats *previous_stats,
+                        struct conn_stats *new_stats)
+{
+    const unsigned long *const cum = (void *) cumulative_stats,
+                        *const prev = (void *) previous_stats;
+    unsigned long *const new = (void *) new_stats;
+    unsigned i;
+
+    for (i = 0; i < sizeof(*new_stats) / sizeof(new[0]); ++i)
+        new[i] = cum[i] - prev[i];
+}
+
+
+#endif
+
+
+const char *
+lsquic_conn_get_sni (struct lsquic_conn *lconn)
+{
+    if (lconn->cn_esf_c && lconn->cn_esf_c->esf_get_sni)
+        return lconn->cn_esf_c->esf_get_sni(lconn->cn_enc_session);
+    else
+        return NULL;
+}
