@@ -1,4 +1,4 @@
-/* Copyright (c) 2017 - 2020 LiteSpeed Technologies Inc.  See LICENSE. */
+/* Copyright (c) 2017 - 2021 LiteSpeed Technologies Inc.  See LICENSE. */
 #include <assert.h>
 #include <stdbool.h>
 #include <string.h>
@@ -162,23 +162,37 @@ const common_cert_t common_cert_set[common_certs_num] = {
 
 static lsquic_str_t *s_ccsbuf;
 
-lsquic_str_t * get_common_certs_hash()
+static int
+match_common_cert (lsquic_str_t * cert, lsquic_str_t * common_set_hashes,
+        uint64_t* out_hash, uint32_t* out_index);
+
+int
+lsquic_crt_init (void)
 {
-    int i;
-    if (s_ccsbuf == NULL)
+    unsigned i;
+
+    s_ccsbuf = lsquic_str_new(NULL, 0);
+    if (!s_ccsbuf)
+        return -1;
+    for (i=0 ;i<common_certs_num; ++i)
     {
-        s_ccsbuf = lsquic_str_new(NULL, 0);
-        for (i=0 ;i<common_certs_num; ++i)
-        {
-            lsquic_str_append(s_ccsbuf, (const char *)&common_cert_set[i].hash, 8);
-        }
+        if (0 != lsquic_str_append(s_ccsbuf, (const char *)&common_cert_set[i].hash, 8))
+            return -1;
     }
+    return 0;
+}
+
+
+lsquic_str_t *
+lsquic_get_common_certs_hash()
+{
     return s_ccsbuf;
 }
 
 
 /* return 0 found, -1 not found */
-int get_common_cert(uint64_t hash, uint32_t index, lsquic_str_t *buf)
+int
+lsquic_get_common_cert(uint64_t hash, uint32_t index, lsquic_str_t *buf)
 {
     int i;
     for (i = 0; i < common_certs_num; i++)
@@ -214,7 +228,8 @@ comp_ls_str (lsquic_str_t * a, const void * b, size_t b_len)
 
 
 /* 0, matched -1, error */
-int match_common_cert(lsquic_str_t * cert, lsquic_str_t * common_set_hashes,
+static int
+match_common_cert (lsquic_str_t * cert, lsquic_str_t * common_set_hashes,
         uint64_t* out_hash, uint32_t* out_index)
 {
     size_t i, j;
@@ -299,12 +314,13 @@ make_zlib_dict_for_entries(cert_entry_t *entries,
 }
 
 
+static
 void get_certs_hash(lsquic_str_t *certs, size_t certs_count, uint64_t *hashs)
 {
     size_t i;
     for(i = 0; i < certs_count; ++i)
     {
-        hashs[i] = fnv1a_64((const uint8_t *)lsquic_str_buf(&certs[i]), lsquic_str_len(&certs[i]));
+        hashs[i] = lsquic_fnv1a_64((const uint8_t *)lsquic_str_buf(&certs[i]), lsquic_str_len(&certs[i]));
     }
 }
 
@@ -331,7 +347,7 @@ static void get_certs_entries(lsquic_str_t **certs, size_t certs_count,
         if (cached_valid)
         {
             cached = false;
-            hash = fnv1a_64((const uint8_t *)lsquic_str_buf(certs[i]), lsquic_str_len(certs[i]));
+            hash = lsquic_fnv1a_64((const uint8_t *)lsquic_str_buf(certs[i]), lsquic_str_len(certs[i]));
 
             for (j = 0; j < (int)lsquic_str_len(client_cached_cert_hashes);
                  j += sizeof(uint64_t))
@@ -362,7 +378,8 @@ static void get_certs_entries(lsquic_str_t **certs, size_t certs_count,
    }
 }
 
-size_t get_entries_size(cert_entry_t *entries, size_t entries_count)
+static size_t
+get_entries_size(cert_entry_t *entries, size_t entries_count)
 {
     size_t i;
     size_t entries_size = 0;
@@ -387,6 +404,7 @@ size_t get_entries_size(cert_entry_t *entries, size_t entries_count)
     return entries_size;
 }
 
+static
 void serialize_cert_entries(uint8_t* out, int *out_len, cert_entry_t *entries,
                             size_t entries_count)
 {
@@ -419,10 +437,11 @@ void serialize_cert_entries(uint8_t* out, int *out_len, cert_entry_t *entries,
 }
 
 
-int get_certs_count(lsquic_str_t *compressed_crt_buf)
+int
+lsquic_get_certs_count (const struct compressed_cert *ccert)
 {
-    char *in = lsquic_str_buf(compressed_crt_buf);
-    char *in_end = in + lsquic_str_len(compressed_crt_buf);
+    const unsigned char *in = ccert->buf;
+    const unsigned char *in_end = in + ccert->len;
     size_t idx = 0;
     uint8_t type_byte;
     
@@ -535,7 +554,7 @@ static int parse_entries(const unsigned char **in_out, const unsigned char *cons
             memcpy(&entry->index, in, sizeof(uint32_t));
             in += sizeof(uint32_t);
 
-            if (0 == get_common_cert(entry->set_hash, entry->index, cert))
+            if (0 == lsquic_get_common_cert(entry->set_hash, entry->index, cert))
                 break;
             else
                 goto err;
@@ -559,13 +578,11 @@ static int parse_entries(const unsigned char **in_out, const unsigned char *cons
 }
 
 
-/* return 0 for OK */
-int compress_certs(lsquic_str_t **certs, size_t certs_count,
+struct compressed_cert *
+lsquic_compress_certs (lsquic_str_t **certs, size_t certs_count,
                    lsquic_str_t *client_common_set_hashes,
-                   lsquic_str_t *client_cached_cert_hashes,
-                   lsquic_str_t *result)
+                   lsquic_str_t *client_cached_cert_hashes)
 {
-    int rv;
     size_t i;
     size_t uncompressed_size = 0, compressed_size = 0 ;
     z_stream z;
@@ -575,10 +592,12 @@ int compress_certs(lsquic_str_t **certs, size_t certs_count,
     uint8_t* out;
     uint32_t tmp_size_32;
     cert_entry_t *entries;
+    struct compressed_cert *ccert;
 
+    ccert = NULL;
     entries = malloc(sizeof(cert_entry_t) * certs_count);
     if (!entries)
-        return -1;
+        return NULL;
 
     dict = lsquic_str_new(NULL, 0);
     if (!dict)
@@ -611,16 +630,18 @@ int compress_certs(lsquic_str_t **certs, size_t certs_count,
     entries_size = get_entries_size(entries, certs_count);
     result_length = entries_size + (uncompressed_size > 0 ? 4 : 0) +
                     compressed_size;
-    lsquic_str_prealloc(result, result_length);
+    ccert = malloc(sizeof(*ccert) + result_length);
+    if (!ccert)
+        goto err;
+    ccert->refcnt = 0;
 
-    out = (unsigned char *)lsquic_str_buf(result);
+    out = ccert->buf;
     serialize_cert_entries(out, &out_len, entries, certs_count);
     out += entries_size;
 
     if (uncompressed_size == 0)
     {
-        lsquic_str_setlen(result, entries_size);
-        rv = 0;
+        ccert->len = entries_size;
         goto cleanup;
     }
     
@@ -651,9 +672,7 @@ int compress_certs(lsquic_str_t **certs, size_t certs_count,
     if (Z_STREAM_END != deflate(&z, Z_FINISH))
         goto err;
 
-    rv = 0;
-    result_length -= z.avail_out;
-    lsquic_str_setlen(result, result_length);
+    ccert->len = result_length - z.avail_out;
 
   cleanup:
     free(entries);
@@ -661,16 +680,19 @@ int compress_certs(lsquic_str_t **certs, size_t certs_count,
         lsquic_str_delete(dict);
     if (uncompressed_size)
         deflateEnd(&z);
-    return rv;
+    return ccert;
 
   err:
-    rv = -1;
+    if (ccert)
+        free(ccert);
+    ccert = NULL;
     goto cleanup;
 }
 
 
 /* 0: ok */
-int decompress_certs(const unsigned char *in, const unsigned char *in_end,
+int
+lsquic_decompress_certs (const unsigned char *in, const unsigned char *in_end,
                      lsquic_str_t *cached_certs, size_t cached_certs_count,
                      lsquic_str_t **out_certs, size_t *out_certs_count)
 {
@@ -684,7 +706,7 @@ int decompress_certs(const unsigned char *in, const unsigned char *in_end,
     z_stream z;
 
     assert(*out_certs_count > 0 && *out_certs_count < 10000
-            && "Call get_certs_count() to get right certificates count first and make enough room for out_certs_count");
+            && "Call lsquic_get_certs_count() to get right certificates count first and make enough room for out_certs_count");
     
     if (count == 0 || count > 10000)
         return -1;
