@@ -1,4 +1,4 @@
-/* Copyright (c) 2017 - 2020 LiteSpeed Technologies Inc.  See LICENSE. */
+/* Copyright (c) 2017 - 2021 LiteSpeed Technologies Inc.  See LICENSE. */
 
 #ifndef __LSQUIC_CRYPTO_H__
 #define __LSQUIC_CRYPTO_H__
@@ -28,14 +28,16 @@ struct x509_st;
 #endif
 
 
-void crypto_init(void);
+void lsquic_crypto_init(void);
 
 
-int export_key_material_simple(unsigned char *ikm, uint32_t ikm_len,
+#ifndef NDEBUG
+int lsquic_export_key_material_simple(unsigned char *ikm, uint32_t ikm_len,
                         unsigned char *salt, int salt_len,
                         char *label, uint32_t label_len,
                         const uint8_t *context, uint32_t context_len,
                         uint8_t *key, uint16_t key_len);
+#endif
 
 int lsquic_export_key_material(const unsigned char *ikm, uint32_t ikm_len,
                         const unsigned char *salt, int salt_len,
@@ -47,39 +49,37 @@ int lsquic_export_key_material(const unsigned char *ikm, uint32_t ikm_len,
                         uint8_t *sub_key,
                         uint8_t *c_hp, uint8_t *s_hp);
 
-void c255_get_pub_key(unsigned char *priv_key, unsigned char pub_key[32]);
-int c255_gen_share_key(unsigned char *priv_key, unsigned char *peer_pub_key, unsigned char *shared_key);
+void lsquic_c255_get_pub_key(unsigned char *priv_key, unsigned char pub_key[32]);
+int lsquic_c255_gen_share_key(unsigned char *priv_key, unsigned char *peer_pub_key, unsigned char *shared_key);
 
 
 
-uint64_t fnv1a_64(const uint8_t * data, int len);
-void fnv1a_64_s(const uint8_t * data, int len, char *md);
-void fnv1a_128_s(const uint8_t * data , int len, uint8_t  *md);
-uint128 fnv1a_128_3(const uint8_t * data1, int len1,
+uint64_t lsquic_fnv1a_64(const uint8_t * data, int len);
+void lsquic_fnv1a_64_s(const uint8_t * data, int len, char *md);
+void lsquic_fnv1a_128_s(const uint8_t * data , int len, uint8_t  *md);
+uint128 lsquic_fnv1a_128_3(const uint8_t * data1, int len1,
                       const uint8_t * data2, int len2,
                       const uint8_t * data3, int len3);
-void serialize_fnv128_short(uint128 v, uint8_t *md);
+void lsquic_serialize_fnv128_short(uint128 v, uint8_t *md);
 
 
 /* Encrypt plaint text to cipher test */
-int aes_aead_enc(struct evp_aead_ctx_st *key,
+int lsquic_aes_aead_enc(struct evp_aead_ctx_st *key,
               const uint8_t *ad, size_t ad_len,
               const uint8_t *nonce, size_t nonce_len, 
               const uint8_t *plain, size_t plain_len,
               uint8_t *cypher, size_t *cypher_len);
 
-int aes_aead_dec(struct evp_aead_ctx_st *key,
+int lsquic_aes_aead_dec(struct evp_aead_ctx_st *key,
               const uint8_t *ad, size_t ad_len,
               const uint8_t *nonce, size_t nonce_len, 
               const uint8_t *cypher, size_t cypher_len,
               uint8_t *plain, size_t *plain_len);
 
 /* 32 bytes client nonce with 4 bytes tm, 8 bytes orbit */
-void gen_nonce_c(unsigned char *buf, uint64_t orbit);
+void lsquic_gen_nonce_c(unsigned char *buf, uint64_t orbit);
 
-struct evp_pkey_st *PEM_to_key(const char *buf, int len);
-
-struct x509_st *bio_to_crt(const void *buf, int len, int type);
+struct x509_st *lsquic_bio_to_crt(const void *buf, int len, int type);
 
 int lshkdf_expand(const unsigned char *prk, const unsigned char *info, int info_len,
                 uint16_t c_key_len, uint8_t *c_key,
@@ -91,14 +91,11 @@ int lshkdf_expand(const unsigned char *prk, const unsigned char *info, int info_
 void lshkdf_extract(const unsigned char *ikm, int ikm_len, const unsigned char *salt,
                   int salt_len, unsigned char *prk);
 
-int gen_prof(const uint8_t *chlo_data, size_t chlo_data_len,
+int lsquic_gen_prof(const uint8_t *chlo_data, size_t chlo_data_len,
              const uint8_t *scfg_data, uint32_t scfg_data_len,
              const struct evp_pkey_st *priv_key, uint8_t *buf, size_t *len);
-int verify_prof0(const uint8_t *chlo_data, size_t chlo_data_len,
-                const uint8_t *scfg_data, uint32_t scfg_data_len,
-                const struct evp_pkey_st *pub_key, const uint8_t *buf, size_t len);
 
-int verify_prof(const uint8_t *chlo_data, size_t chlo_data_len, struct lsquic_str * scfg,
+int lsquic_verify_prof(const uint8_t *chlo_data, size_t chlo_data_len, struct lsquic_str * scfg,
                 const struct evp_pkey_st *pub_key, const uint8_t *buf, size_t len);
 
 
