@@ -1,4 +1,4 @@
-/* Copyright (c) 2017 - 2019 LiteSpeed Technologies Inc.  See LICENSE. */
+/* Copyright (c) 2017 - 2021 LiteSpeed Technologies Inc.  See LICENSE. */
 #ifndef LSQUIC_ENC_SESS_H
 #define LSQUIC_ENC_SESS_H 1
 
@@ -15,6 +15,8 @@ struct ssl_stream_method_st;
 struct ssl_st;
 struct sockaddr;
 struct conn_cid_elem;
+struct lsquic_engine_settings;
+enum lsquic_version;
 
 #define DNONC_LENGTH 32
 #define SRST_LENGTH 16
@@ -51,8 +53,6 @@ enum handshake_error            /* TODO: rename this enum */
     HS_SHLO = 0,
     HS_1RTT = 1,
     HS_SREJ = 2,
-    HS_DELAYED = 3,
-    HS_PK_OFFLOAD = 4,
 };
 
 #ifndef LSQUIC_KEEP_ENC_SESS_HISTORY
@@ -102,6 +102,9 @@ struct enc_session_funcs_common
     const char *
     (*esf_get_sni) (enc_session_t *);
 
+    /* Need to pass lconn in encrypt and decrypt methods because enc_session
+     * is allowed to be NULL for gQUIC.
+     */
     enum enc_packout
     (*esf_encrypt_packet) (enc_session_t *, const struct lsquic_engine_public *,
         struct lsquic_conn *, struct lsquic_packet_out *);
@@ -117,10 +120,19 @@ struct enc_session_funcs_common
     (*esf_verify_reset_token) (enc_session_t *, const unsigned char *, size_t);
 
     int
-    (*esf_did_zero_rtt_succeed) (enc_session_t *);
+    (*esf_did_sess_resume_succeed) (enc_session_t *);
 
     int
-    (*esf_is_zero_rtt_enabled) (enc_session_t *);
+    (*esf_is_sess_resume_enabled) (enc_session_t *);
+
+    void
+    (*esf_set_conn) (enc_session_t *, struct lsquic_conn *);
+
+    /* Optional.  This function gets called after packets are encrypted,
+     * batched, and are about to be sent.
+     */
+    void
+    (*esf_flush_encryption) (enc_session_t *);
 
     unsigned
     esf_tag_len;
@@ -150,7 +162,8 @@ struct enc_session_funcs_gquic
 
     /* Create server session */
     enc_session_t *
-    (*esf_create_server) (lsquic_cid_t cid, const struct lsquic_engine_public *);
+    (*esf_create_server) (struct lsquic_conn *,
+                        lsquic_cid_t cid, struct lsquic_engine_public *);
 
     /* out_len should have init value as the max length of out */
     enum handshake_error
@@ -165,7 +178,7 @@ struct enc_session_funcs_gquic
 #ifndef NDEBUG
     /* Need to expose this function for testing */
     int (*esf_determine_diversification_key) (enc_session_t *,
-                              uint8_t *diversification_nonce, int is_client);
+                              uint8_t *diversification_nonce);
 #endif
 
     const char *
@@ -204,15 +217,11 @@ struct enc_session_funcs_gquic
     (*esf_get_dec_key_nonce_f) (enc_session_t *);
 #endif /* !defined(NDEBUG) */
 
-#if LSQUIC_ENABLE_HANDSHAKE_DISABLE
-    void
-    (*esf_set_handshake_completed) (enc_session_t *);
-#endif
-
     /* Create client session */
     enc_session_t *
-    (*esf_create_client) (const char *domain, lsquic_cid_t cid,
-                                    const struct lsquic_engine_public *,
+    (*esf_create_client) (struct lsquic_conn *, const char *domain,
+                            lsquic_cid_t cid,
+                                    struct lsquic_engine_public *,
                                     const unsigned char *, size_t);
 
     /* -1 error, 0, OK, response in `buf' */
@@ -227,23 +236,17 @@ struct enc_session_funcs_gquic
     size_t
     (*esf_mem_used)(enc_session_t *);
 
-    /* Zero-rtt serialization needs the knowledge of the QUIC version, that's
-     * why there is a separate method for thus.  Plus, we want to be able to
-     * call it after the "handshake is done" callback is called.
+    /* Session resumption serialization needs the knowledge of the QUIC
+     * version, that's why there is a separate method for thus.  Plus, we
+     * want to be able to call it after the "handshake is done" callback
+     * is called.
      */
-    void (*esf_maybe_dispatch_zero_rtt) (enc_session_t *,
-            struct lsquic_conn *conn,
+    void (*esf_maybe_dispatch_sess_resume) (enc_session_t *,
             void (*cb)(struct lsquic_conn *, const unsigned char *, size_t));
 
     void (*esf_reset_cid) (enc_session_t *, const lsquic_cid_t *);
 };
 
-enum iquic_handshake_status {
-    IHS_WANT_READ,
-    IHS_WANT_WRITE,
-    IHS_STOP,
-};
-
 struct crypto_stream_if
 {
     ssize_t     (*csi_write) (void *stream, const void *buf, size_t len);
@@ -264,7 +267,7 @@ struct enc_session_funcs_iquic
                            const struct ver_neg *, void *(crypto_streams)[4],
                            const struct crypto_stream_if *,
                            const unsigned char *, size_t,
-                           struct lsquic_alarmset *);
+                           struct lsquic_alarmset *, unsigned, void*);
 
     void
     (*esfi_destroy) (enc_session_t *);
@@ -279,12 +282,12 @@ struct enc_session_funcs_iquic
     (*esfi_reset_dcid) (enc_session_t *, const struct lsquic_cid *,
                                                 const struct lsquic_cid *);
 
+    void
+    (*esfi_set_iscid) (enc_session_t *, const struct lsquic_packet_in *);
+
     int
     (*esfi_init_server) (enc_session_t *);
 
-    void
-    (*esfi_set_conn) (enc_session_t *, struct lsquic_conn *);
-
     void
     (*esfi_set_streams) (enc_session_t *, void *(crypto_streams)[4],
                            const struct crypto_stream_if *);
@@ -294,14 +297,22 @@ struct enc_session_funcs_iquic
                                                     const struct lsquic_cid *,
                            void *(crypto_streams)[4],
                            const struct crypto_stream_if *,
-                           const struct lsquic_cid *odcid);
+                           const struct lsquic_cid *odcid,
+                           const struct lsquic_cid *iscid);
 
     void
     (*esfi_shake_stream)(enc_session_t *, struct lsquic_stream *,
                          const char *);
 
     void
-    (*esfi_1rtt_acked)(enc_session_t *);
+    (*esfi_handshake_confirmed)(enc_session_t *);
+
+    int
+    (*esfi_in_init)(enc_session_t *);
+
+    int
+    (*esfi_data_in)(enc_session_t *, enum enc_level,
+                                            const unsigned char *, size_t);
 };
 
 extern
@@ -309,6 +320,13 @@ extern
 const
 #endif
 struct enc_session_funcs_common lsquic_enc_session_common_gquic_1;
+
+extern
+#ifdef NDEBUG
+const
+#endif
+struct enc_session_funcs_common lsquic_enc_session_common_gquic_2;
+
 extern const struct enc_session_funcs_common lsquic_enc_session_common_ietf_v1;
 
 extern
@@ -320,8 +338,12 @@ struct enc_session_funcs_gquic lsquic_enc_session_gquic_gquic_1;
 extern const struct enc_session_funcs_iquic lsquic_enc_session_iquic_ietf_v1;
 
 #define select_esf_common_by_ver(ver) ( \
-    ver == LSQVER_ID23 ? &lsquic_enc_session_common_ietf_v1 : \
+    ver == LSQVER_ID27 ? &lsquic_enc_session_common_ietf_v1 : \
+    ver == LSQVER_ID28 ? &lsquic_enc_session_common_ietf_v1 : \
+    ver == LSQVER_ID29 ? &lsquic_enc_session_common_ietf_v1 : \
+    ver == LSQVER_ID32 ? &lsquic_enc_session_common_ietf_v1 : \
     ver == LSQVER_VERNEG ? &lsquic_enc_session_common_ietf_v1 : \
+    ver == LSQVER_050 ? &lsquic_enc_session_common_gquic_2 : \
     &lsquic_enc_session_common_gquic_1 )
 
 #define select_esf_gquic_by_ver(ver) ( \
@@ -340,11 +362,17 @@ extern const struct lsquic_stream_if lsquic_mini_cry_sm_if;
 #define ALERT_NO_APPLICATION_PROTOCOL 120
 
 enum lsquic_version
-lsquic_zero_rtt_version (const unsigned char *, size_t);
+lsquic_sess_resume_version (const unsigned char *, size_t);
 
 /* This is seems to be true for all of the ciphers used by IETF QUIC.
  * XXX: Perhaps add a check?
  */
 #define IQUIC_TAG_LEN 16
 
+/* Return number of bytes written to `buf' or -1 on error */
+int
+lsquic_enc_sess_ietf_gen_quic_ctx (
+                const struct lsquic_engine_settings *settings,
+                enum lsquic_version version, unsigned char *buf, size_t bufsz);
+
 #endif
