@@ -1,4 +1,4 @@
-/* Copyright (c) 2017 - 2019 LiteSpeed Technologies Inc.  See LICENSE. */
+/* Copyright (c) 2017 - 2021 LiteSpeed Technologies Inc.  See LICENSE. */
 /*
  * lsquic_frame_reader.c -- Read HTTP frames from stream
  */
@@ -380,7 +380,7 @@ prepare_for_payload (struct lsquic_frame_reader *fr)
             LSQ_WARN("cannot allocate %u bytes for header block",
                                                     fr->fr_header_block_sz);
             fr->fr_callbacks->frc_on_error(fr->fr_cb_ctx, stream_id,
-                                                                FR_ERR_NOMEM);
+                                                        FR_ERR_OTHER_ERROR);
             return -1;
         }
         fr->fr_header_block = header_block;
@@ -430,7 +430,7 @@ prepare_for_payload (struct lsquic_frame_reader *fr)
         LSQ_INFO("headers are too large (%u bytes), skipping",
             fr->fr_state.payload_length);
         fr->fr_callbacks->frc_on_error(fr->fr_cb_ctx, stream_id,
-                                                FR_ERR_HEADERS_TOO_LARGE);
+                                                FR_ERR_BAD_HEADER);
         /* fallthru */
     continue_skipping:
     default:
@@ -509,32 +509,92 @@ skip_headers_padding (struct lsquic_frame_reader *fr)
 }
 
 
-static int
+static struct lsquic_stream *
+find_target_stream (const struct lsquic_frame_reader *fr)
+{
+    lsquic_stream_id_t stream_id;
+    struct lsquic_conn *lconn;
+
+    stream_id = fr_get_stream_id(fr);
+    lconn = lsquic_stream_conn(fr->fr_stream);
+    if (lconn->cn_if->ci_get_stream_by_id)
+        return lconn->cn_if->ci_get_stream_by_id(lconn, stream_id);
+
+    return NULL;
+}
+
+
+static void
+skip_headers (struct lsquic_frame_reader *fr)
+{
+    const unsigned char *comp, *end;
+    void *buf;
+    int s;
+    struct lsxpack_header xhdr;
+    const size_t buf_len = 64 * 1024;
+
+    buf = malloc(buf_len);
+    if (!buf)
+    {
+        fr->fr_callbacks->frc_on_error(fr->fr_cb_ctx, fr_get_stream_id(fr),
+                                                        FR_ERR_OTHER_ERROR);
+        goto end;
+    }
+
+    comp = fr->fr_header_block;
+    end = comp + fr->fr_header_block_sz;
+    while (comp < end)
+    {
+        lsxpack_header_prepare_decode(&xhdr, buf, 0, buf_len);
+        s = lshpack_dec_decode(fr->fr_hdec, &comp, end, &xhdr);
+        if (s != 0)
+        {
+            fr->fr_callbacks->frc_on_error(fr->fr_cb_ctx, fr_get_stream_id(fr),
+                                                            FR_ERR_OTHER_ERROR);
+            break;
+        }
+    }
+
+  end:
+    if (buf)
+        free(buf);
+}
+
+
+static void
 decode_and_pass_payload (struct lsquic_frame_reader *fr)
 {
     struct headers_state *hs = &fr->fr_state.by_type.headers_state;
     const unsigned char *comp, *end;
     enum frame_reader_error err;
     int s;
-    uint32_t name_idx;
-    lshpack_strlen_t name_len, val_len;
-    char *buf;
     uint32_t stream_id32;
     struct uncompressed_headers *uh = NULL;
     void *hset = NULL;
+    struct lsxpack_header *hdr = NULL;
+    size_t req_space = 0;
+    lsquic_stream_t *target_stream = NULL;
 
-    buf = lsquic_mm_get_16k(fr->fr_mm);
-    if (!buf)
+    if (!(fr->fr_flags & FRF_SERVER))
     {
-        err = FR_ERR_NOMEM;
-        goto stream_error;
+        target_stream = find_target_stream(fr);
+        /* If the response is for a stream that cannot be found, one of two
+         * things is true: a) the stream has been closed or b) this is an
+         * error.  If (a), we discard this header block.  We choose to do the
+         * same for (b) instead of erroring out for the sake of simplicity.
+         * There is no way to exploit this behavior.
+         */
+        if (!target_stream)
+        {
+            skip_headers(fr);
+            return;
+        }
     }
-
-    hset = fr->fr_hsi_if->hsi_create_header_set(fr->fr_hsi_ctx,
+    hset = fr->fr_hsi_if->hsi_create_header_set(fr->fr_hsi_ctx, target_stream,
                             READER_PUSH_PROMISE == fr->fr_state.reader_type);
     if (!hset)
     {
-        err = FR_ERR_NOMEM;
+        err = FR_ERR_OTHER_ERROR;
         goto stream_error;
     }
 
@@ -543,40 +603,54 @@ decode_and_pass_payload (struct lsquic_frame_reader *fr)
 
     while (comp < end)
     {
-        s = lshpack_dec_decode(fr->fr_hdec, &comp, end,
-                    buf, buf + 16 * 1024, &name_len, &val_len, &name_idx);
+  prepare:
+        hdr = fr->fr_hsi_if->hsi_prepare_decode(hset, hdr, req_space);
+        if (!hdr)
+        {
+            err = FR_ERR_OTHER_ERROR;
+            goto stream_error;
+        }
+        s = lshpack_dec_decode(fr->fr_hdec, &comp, end, hdr);
         if (s == 0)
         {
-            if (name_idx > 61   /* XXX 61 */)
-                name_idx = 0;   /* Work around bug in ls-hpack */
-            err = (enum frame_reader_error)
-                fr->fr_hsi_if->hsi_process_header(hset, name_idx, buf,
-                                        name_len, buf + name_len, val_len);
-            if (err == 0)
+            s = fr->fr_hsi_if->hsi_process_header(hset, hdr);
+            if (s == 0)
             {
 #if LSQUIC_CONN_STATS
-                fr->fr_conn_stats->in.headers_uncomp += name_len + val_len;
+                fr->fr_conn_stats->in.headers_uncomp += hdr->name_len +
+                                                        hdr->val_len;
 #endif
+                req_space = 0;
+                hdr = NULL;
                 continue;
             }
+            else if (s > 0)
+                err = FR_ERR_BAD_HEADER;
+            else
+                err = FR_ERR_OTHER_ERROR;
+        }
+        else if (s == LSHPACK_ERR_MORE_BUF)
+        {
+            req_space = hdr->val_len;
+            goto prepare;
         }
         else
             err = FR_ERR_DECOMPRESS;
         goto stream_error;
     }
     assert(comp == end);
-    lsquic_mm_put_16k(fr->fr_mm, buf);
-    buf = NULL;
 
-    err = (enum frame_reader_error)
-        fr->fr_hsi_if->hsi_process_header(hset, 0, 0, 0, 0, 0);
-    if (err)
+    s = fr->fr_hsi_if->hsi_process_header(hset, NULL);
+    if (s != 0)
+    {
+        err = s < 0 ? FR_ERR_OTHER_ERROR : FR_ERR_BAD_HEADER;
         goto stream_error;
+    }
 
     uh = calloc(1, sizeof(*uh));
     if (!uh)
     {
-        err = FR_ERR_NOMEM;
+        err = FR_ERR_OTHER_ERROR;
         goto stream_error;
     }
 
@@ -613,18 +687,13 @@ decode_and_pass_payload (struct lsquic_frame_reader *fr)
     fr->fr_conn_stats->in.headers_comp += fr->fr_header_block_sz;
 #endif
 
-    return 0;
+    return;
 
   stream_error:
     LSQ_INFO("%s: stream error %u", __func__, err);
     if (hset)
         fr->fr_hsi_if->hsi_discard_header_set(hset);
-    if (uh)
-        free(uh);
-    if (buf)
-        lsquic_mm_put_16k(fr->fr_mm, buf);
     fr->fr_callbacks->frc_on_error(fr->fr_cb_ctx, fr_get_stream_id(fr), err);
-    return 0;
 }
 
 
@@ -654,13 +723,12 @@ read_headers_block_fragment (struct lsquic_frame_reader *fr)
     if (hs->nread == payload_length &&
                 (fr->fr_state.header.hfh_flags & HFHF_END_HEADERS))
     {
-        int rv = decode_and_pass_payload(fr);
+        decode_and_pass_payload(fr);
         free(fr->fr_header_block);
         fr->fr_header_block = NULL;
-        return rv;
     }
-    else
-        return 0;
+
+    return 0;
 }
 
 
@@ -794,20 +862,14 @@ read_contin (struct lsquic_frame_reader *fr)
     {
         if (fr->fr_state.header.hfh_flags & HFHF_END_HEADERS)
         {
-            int rv = decode_and_pass_payload(fr);
+            decode_and_pass_payload(fr);
             free(fr->fr_header_block);
             fr->fr_header_block = NULL;
-            reset_state(fr);
-            return rv;
-        }
-        else
-        {
-            reset_state(fr);
-            return 0;
         }
+        reset_state(fr);
     }
-    else
-        return 0;
+
+    return 0;
 }
 
 
