@@ -1,16 +1,15 @@
-/* Copyright (c) 2017 - 2019 LiteSpeed Technologies Inc.  See LICENSE. */
+/* Copyright (c) 2017 - 2021 LiteSpeed Technologies Inc.  See LICENSE. */
 /*
- * lsquic_min_heap.h -- Min-heap for connections
+ * lsquic_min_heap.h -- Min-heap for pointers
  */
 
 #ifndef LSQUIC_MIN_HEAP_H
 #define LSQUIC_MIN_HEAP_H 1
 
-struct lsquic_conn;
 
 struct min_heap_elem
 {
-    struct lsquic_conn  *mhe_conn;
+    void                *mhe_item;
     uint64_t             mhe_val;
 };
 
@@ -24,13 +23,19 @@ struct min_heap
 
 
 void
-lsquic_mh_insert (struct min_heap *, struct lsquic_conn *conn, uint64_t val);
+lsquic_mh_insert (struct min_heap *, void *item, uint64_t val);
 
-struct lsquic_conn *
+void *
 lsquic_mh_pop (struct min_heap *);
 
+#define lsquic_mh_peek(heap) ((heap)->mh_elems[0].mhe_item)
+
 #define lsquic_mh_count(heap) (+(heap)->mh_nelem)
 
 #define lsquic_mh_nalloc(heap) (+(heap)->mh_nalloc)
 
+#define MHE_PARENT(i) ((i - 1) / 2)
+#define MHE_LCHILD(i) (2 * i + 1)
+#define MHE_RCHILD(i) (2 * i + 2)
+
 #endif
