@@ -1,4 +1,4 @@
-/* Copyright (c) 2017 - 2020 LiteSpeed Technologies Inc.  See LICENSE. */
+/* Copyright (c) 2017 - 2021 LiteSpeed Technologies Inc.  See LICENSE. */
 /*
  * lsquic_mini_conn.c -- Mini connection.
  *
@@ -21,9 +21,11 @@
 #include <assert.h>
 #include <errno.h>
 #include <inttypes.h>
+#include <limits.h>
 #include <stdlib.h>
 #include <string.h>
 #include <sys/queue.h>
+#include <time.h>
 
 #include "lsquic.h"
 #include "lsquic_int_types.h"
@@ -95,7 +97,7 @@ process_deferred_packets (struct mini_conn *mc);
 
 /* If this is not true, highest_bit_set() may be broken */
 typedef char packno_set_is_unsigned_long[
-        (sizeof(unsigned long long) == sizeof(mconn_packno_set_t)) - 1];
+        sizeof(unsigned long long) == sizeof(mconn_packno_set_t) ? 1 : -1 ];
 
 static unsigned
 highest_bit_set (unsigned long long sz)
@@ -185,7 +187,7 @@ packet_in_is_ok (enum lsquic_version version,
 
 
 lsquic_conn_t *
-mini_conn_new (struct lsquic_engine_public *enp,
+lsquic_mini_conn_new (struct lsquic_engine_public *enp,
                const struct lsquic_packet_in *packet_in,
                enum lsquic_version version)
 {
@@ -290,6 +292,8 @@ process_ack_frame (struct mini_conn *mc, lsquic_packet_in_t *packet_in,
         LSQ_DEBUG("Ignore old ack (max %u)", mc->mc_max_ack_packno);
         return parsed_len;
     }
+    if (packet_in->pi_packno <= UCHAR_MAX)
+        mc->mc_max_ack_packno = packet_in->pi_packno;
 
     /* Verify ACK frame and update list of acked packet numbers: */
     for (n = 0; n < acki->n_ranges; ++n)
@@ -825,7 +829,7 @@ mini_stream_read (void *stream, void *buf, size_t len, int *reached_fin)
 
 /* Wrapper to throw out reached_fin */
 static size_t
-mini_stream_read_for_crypto (void *stream, void *buf, size_t len)
+mini_stream_read_for_crypto (void *stream, void *buf, size_t len, int *fin)
 {
     size_t retval;
     int reached_fin;
@@ -883,7 +887,8 @@ allocate_packet_out (struct mini_conn *mc, const unsigned char *nonce)
         return NULL;
     }
     packet_out = lsquic_packet_out_new(&mc->mc_enpub->enp_mm, NULL, 1,
-                &mc->mc_conn, GQUIC_PACKNO_LEN_1, NULL, nonce, &mc->mc_path);
+                &mc->mc_conn, GQUIC_PACKNO_LEN_1, NULL, nonce, &mc->mc_path,
+                HETY_NOT_SET);
     if (!packet_out)
     {
         LSQ_WARN("could not allocate packet: %s", strerror(errno));
@@ -968,7 +973,7 @@ to_packet_Q050plus (struct mini_conn *mc, struct mini_stream_ctx *ms_ctx,
     cur_off = ms_ctx->off;
     len = mc->mc_conn.cn_pf->pf_gen_crypto_frame(
             packet_out->po_data + packet_out->po_data_sz,
-            lsquic_packet_out_avail(packet_out), mc->mc_write_off,
+            lsquic_packet_out_avail(packet_out), 0, mc->mc_write_off, 0,
             mini_stream_size(ms_ctx), mini_stream_read_for_crypto, ms_ctx);
     if (len < 0)
     {
@@ -1184,7 +1189,7 @@ continue_handshake (struct mini_conn *mc)
     MCHIST_APPEND(mc, he == DATA_NOT_ENOUGH ? MCHE_HANDLE_NOT_ENOUGH :
                       he == HS_SHLO         ? MCHE_HANDLE_SHLO :
                       he == HS_1RTT         ? MCHE_HANDLE_1RTT :
-                      he == HS_2RTT         ? MCHE_HANDLE_2RTT :
+                      he == HS_SREJ         ? MCHE_HANDLE_SREJ :
                       he == HS_ERROR        ? MCHE_HANDLE_ERROR :
                                               MCHE_HAHDLE_UNKNOWN);
 
@@ -1700,6 +1705,7 @@ process_deferred_packets (struct mini_conn *mc)
 
 
 #if LSQUIC_RECORD_INORD_HIST
+/* FIXME This does not work for Q050, where 0 is a valid packet number. */
 /* Packet number is encoded as a sequence of 1-bits and stored in mc_inord_hist
  * separated by 0 bits.  For example, sequence of packet numbers 3, 2, 1 would
  * be encoded as (starting with LSB) 1110110100000000...  This is not the most
@@ -1730,6 +1736,26 @@ record_inord_packno (struct mini_conn *mc, lsquic_packno_t packno)
 }
 
 
+#if __GNUC__
+#   define ctz __builtin_ctzll
+#else
+static unsigned
+ctz (unsigned long long x)
+{
+    unsigned n = 0;
+    if (0 == (x & ((1ULL << 32) - 1))) { n += 32; x >>= 32; }
+    if (0 == (x & ((1ULL << 16) - 1))) { n += 16; x >>= 16; }
+    if (0 == (x & ((1ULL <<  8) - 1))) { n +=  8; x >>=  8; }
+    if (0 == (x & ((1ULL <<  4) - 1))) { n +=  4; x >>=  4; }
+    if (0 == (x & ((1ULL <<  2) - 1))) { n +=  2; x >>=  2; }
+    if (0 == (x & ((1ULL <<  1) - 1))) { n +=  1; x >>=  1; }
+    return n;
+}
+
+
+#endif
+
+
 static void
 inord_to_str (const struct mini_conn *mc, char *buf, size_t bufsz)
 {
@@ -1746,7 +1772,7 @@ inord_to_str (const struct mini_conn *mc, char *buf, size_t bufsz)
         hist = mc->mc_inord_hist[n];
         while (hist)
         {
-            n_trail = __builtin_ctzll(~hist);
+            n_trail = ctz(~hist);
             nw = snprintf(buf + off, bufsz - off,
                 /* No spaces are included on purpose: this makes it a single
                  * field and thus easy to process log using standard command-
@@ -1852,6 +1878,7 @@ mini_conn_ci_Q050_packet_in (struct lsquic_conn *lconn,
         return;
     }
 
+
     if (!mc->mc_conn.cn_enc_session)
     {
         mc->mc_conn.cn_enc_session =
@@ -1898,12 +1925,13 @@ mini_conn_ci_Q050_packet_in (struct lsquic_conn *lconn,
 
 
 static struct lsquic_packet_out *
-mini_conn_ci_next_packet_to_send (struct lsquic_conn *lconn, size_t size)
+mini_conn_ci_next_packet_to_send (struct lsquic_conn *lconn,
+                                        const struct to_coal *to_coal_UNUSED)
 {
     struct mini_conn *mc = (struct mini_conn *) lconn;
     lsquic_packet_out_t *packet_out;
 
-    assert(0 == size);
+    assert(NULL == to_coal_UNUSED);
     TAILQ_FOREACH(packet_out, &mc->mc_packets_out, po_next)
     {
         if (packet_out->po_flags & PO_SENT)
@@ -1999,7 +2027,7 @@ mini_conn_ci_destroy (struct lsquic_conn *lconn)
             mc->mc_received_packnos, mc->mc_sent_packnos, mc->mc_lost_packnos,
             mc->mc_deferred_packnos, still_deferred,
             mc->mc_dropped_packnos, in_flight, mc->mc_acked_packnos,
-            mc->mc_error_code, mc->mc_n_ticks, mc->mc_conn.cn_pack_size,
+            mc->mc_error_code, mc->mc_n_ticks, mc->mc_path.np_pack_size,
             lsquic_time_now() - mc->mc_created,
             lsquic_ver2str[mc->mc_conn.cn_version],
             (int) hist_idx, mc->mc_hist_buf);
@@ -2023,13 +2051,16 @@ mini_conn_ci_destroy (struct lsquic_conn *lconn)
             mc->mc_received_packnos, mc->mc_sent_packnos, mc->mc_lost_packnos,
             mc->mc_deferred_packnos, still_deferred,
             mc->mc_dropped_packnos, in_flight, mc->mc_acked_packnos,
-            mc->mc_error_code, mc->mc_n_ticks, mc->mc_conn.cn_pack_size,
+            mc->mc_error_code, mc->mc_n_ticks, mc->mc_path.np_pack_size,
             lsquic_time_now() - mc->mc_created,
             lsquic_ver2str[mc->mc_conn.cn_version],
             (int) (sizeof(mc->mc_hist_buf) - hist_idx),
             mc->mc_hist_buf + hist_idx, (int) hist_idx, mc->mc_hist_buf);
 #else
-    LSQ_LOG(log_level, "destroyed.  Diagnostics: conn flags: 0x%X, "
+    if (LSQ_LOG_ENABLED(log_level))
+        lsquic_logger_log2(log_level, LSQUIC_LOGGER_MODULE,
+                                   LSQUIC_LOG_CONN_ID,
+        "destroyed.  Diagnostics: conn flags: 0x%X, "
         "mc flags: 0x%X, "
 #if LSQUIC_RECORD_INORD_HIST
         "incoming-history (trunc: %d) %s, "
@@ -2234,8 +2265,8 @@ static const struct conn_iface mini_conn_iface_standard_Q050 = {
 
 typedef char largest_recv_holds_at_least_16_seconds[
     ((1 << (sizeof(((struct mini_conn *) 0)->mc_largest_recv) * 8)) / 1000000
-                                                                    >= 16) - 1];
+                                                                    >= 16) ? 1 : -1];
 
 typedef char max_lifespan_smaller_than_largest_recv[
     ((1 << (sizeof(((struct mini_conn *) 0)->mc_largest_recv) * 8)) >
-                                           MAX_MINI_CONN_LIFESPAN_IN_USEC) - 1];
+                                           MAX_MINI_CONN_LIFESPAN_IN_USEC) ? 1 : -1];
