@@ -1,4 +1,4 @@
-/* Copyright (c) 2017 - 2020 LiteSpeed Technologies Inc.  See LICENSE. */
+/* Copyright (c) 2017 - 2021 LiteSpeed Technologies Inc.  See LICENSE. */
 #ifndef LSQUIC_PARSE_H
 #define LSQUIC_PARSE_H 1
 
@@ -58,9 +58,6 @@ typedef lsquic_time_t
 /* gsf_: generate stream frame */
 typedef size_t (*gsf_read_f) (void *stream, void *buf, size_t len, int *fin);
 
-/* gcf_: generate CRYPTO frame */
-typedef size_t (*gcf_read_f) (void *stream, void *buf, size_t len);
-
 /* This structure contains functions that parse and generate packets and
  * frames in version-specific manner.  To begin with, there is difference
  * between GQUIC's little-endian (Q038 and lower) and big-endian formats
@@ -71,7 +68,9 @@ struct parse_funcs
     /* Return buf length */
     int
     (*pf_gen_reg_pkt_header) (const struct lsquic_conn *,
-                const struct lsquic_packet_out *, unsigned char *, size_t);
+                const struct lsquic_packet_out *, unsigned char *, size_t,
+                /* In Q050 and IETF QUIC, these are set: */
+                unsigned *packno_off, unsigned *packno_len);
     void
     (*pf_parse_packet_in_finish) (struct lsquic_packet_in *packet_in,
                                                 struct packin_parse_state *);
@@ -83,19 +82,34 @@ struct parse_funcs
      * exception is -1, which is a generic error code, as we always need
      * more than 1 byte to write a STREAM frame.
      */
+    /* pf_gen_stream_frame and pf_gen_crypto_frame must be adjacent so that
+     * they can be cast to an array.
+     */
     int
     (*pf_gen_stream_frame) (unsigned char *buf, size_t bufsz,
                             lsquic_stream_id_t stream_id, uint64_t offset,
                             int fin, size_t size, gsf_read_f, void *stream);
+    /* The two "UNUSED" parameters are here so that it matches
+     * pf_gen_stream_frame.
+     */
+    int
+    (*pf_gen_crypto_frame) (unsigned char *buf, size_t bufsz,
+                            lsquic_stream_id_t UNUSED_1, uint64_t offset,
+                            int UNUSED_2, size_t size, gsf_read_f, void *stream);
+    /* pf_parse_stream_frame and pf_parse_crypto_frame must be adjacent so that
+     * they can be cast to an array.
+     */
     int
     (*pf_parse_stream_frame) (const unsigned char *buf, size_t rem_packet_sz,
                                                     struct stream_frame *);
     int
     (*pf_parse_crypto_frame) (const unsigned char *buf, size_t rem_packet_sz,
                                                     struct stream_frame *);
+    /* Return true if STREAM frame extends to the end of the packet and thus
+     * does not contain a Length field (no update).
+     */
     int
-    (*pf_gen_crypto_frame) (unsigned char *buf, size_t bufsz, uint64_t offset,
-                            size_t size, gcf_read_f, void *stream);
+    (*pf_dec_stream_frame_size) (unsigned char *buf, size_t new_size);
     int
     (*pf_parse_ack_frame) (const unsigned char *buf, size_t buf_len,
                                     struct ack_info *ack_info, uint8_t exp);
@@ -208,7 +222,7 @@ struct parse_funcs
      */
     size_t
     (*pf_packout_max_header_size) (const struct lsquic_conn *,
-                                    enum packet_out_flags, size_t dcid_len);
+                    enum packet_out_flags, size_t dcid_len, enum header_type);
 
     enum packno_bits
     (*pf_calc_packno_bits) (lsquic_packno_t packno,
@@ -216,11 +230,6 @@ struct parse_funcs
     unsigned
     (*pf_packno_bits2len) (enum packno_bits);
 
-    /* Used by IETF QUIC and gQUIC >= Q050 */
-    void
-    (*pf_packno_info) (const struct lsquic_conn *,
-        const struct lsquic_packet_out *, unsigned *packno_off,
-        unsigned *packno_len);
     int
     (*pf_parse_max_data) (const unsigned char *, size_t, uint64_t *);
     int
@@ -303,17 +312,26 @@ struct parse_funcs
     (*pf_handshake_done_frame_size) (void);
     int
     (*pf_gen_ack_frequency_frame) (unsigned char *buf, size_t buf_len,
-        uint64_t seqno, uint64_t pack_tol, uint64_t upd_mad);
+        uint64_t seqno, uint64_t pack_tol, uint64_t upd_mad, int ignore);
     int
     (*pf_parse_ack_frequency_frame) (const unsigned char *buf, size_t buf_len,
-        uint64_t *seqno, uint64_t *pack_tol, uint64_t *upd_mad);
+        uint64_t *seqno, uint64_t *pack_tol, uint64_t *upd_mad, int *ignore);
     unsigned
     (*pf_ack_frequency_frame_size) (uint64_t seqno, uint64_t pack_tol,
-        uint64_t upd_mad);
+        uint64_t upd_mad /* Don't need to pass `ignore' */);
     int
     (*pf_gen_timestamp_frame) (unsigned char *buf, size_t buf_len, uint64_t);
     int
     (*pf_parse_timestamp_frame) (const unsigned char *buf, size_t, uint64_t *);
+    int
+    (*pf_parse_datagram_frame) (const unsigned char *buf, size_t, const void **,
+                                                                    size_t *);
+    int
+    (*pf_gen_datagram_frame) (unsigned char *, size_t bufsz, size_t min_sz,
+        size_t max_sz, ssize_t (*)(struct lsquic_conn *, void *, size_t),
+        struct lsquic_conn *);
+    unsigned
+    (*pf_datagram_frame_size) (size_t);
 };
 
 
@@ -354,7 +372,7 @@ lsquic_parse_frame_type_gquic_Q035_thru_Q046 (const unsigned char *, size_t);
 extern const enum quic_frame_type lsquic_iquic_byte2type[0x40];
 
 size_t
-calc_stream_frame_header_sz_gquic (lsquic_stream_id_t stream_id,
+lsquic_calc_stream_frame_header_sz_gquic (lsquic_stream_id_t stream_id,
                                                     uint64_t offset, unsigned);
 
 size_t
@@ -363,7 +381,7 @@ lsquic_gquic_packout_size (const struct lsquic_conn *,
 
 size_t
 lsquic_gquic_packout_header_size (const struct lsquic_conn *conn,
-                                enum packet_out_flags flags, size_t unused);
+                enum packet_out_flags flags, size_t unused, enum header_type);
 
 size_t
 lsquic_gquic_po_header_sz (enum packet_out_flags flags);
