@@ -1,4 +1,4 @@
-/* Copyright (c) 2017 - 2020 LiteSpeed Technologies Inc.  See LICENSE. */
+/* Copyright (c) 2017 - 2021 LiteSpeed Technologies Inc.  See LICENSE. */
 /*
  * lsquic_qdec_hdl.c -- QPACK decoder streams handler
  */
@@ -12,6 +12,7 @@
 
 #include "lsquic.h"
 #include "lsquic_types.h"
+#include "lsxpack_header.h"
 #include "lsquic_int_types.h"
 #include "lsquic_sfcw.h"
 #include "lsquic_varint.h"
@@ -26,13 +27,39 @@
 #include "lsquic_engine_public.h"
 #include "lsquic_headers.h"
 #include "lsquic_conn.h"
+#include "lsquic_conn_flow.h"
+#include "lsquic_rtt.h"
+#include "lsquic_conn_public.h"
+#include "lsquic_hq.h"
+#include "lsquic_parse.h"
+#include "lsquic_qpack_exp.h"
+#include "lsquic_util.h"
 
 #define LSQUIC_LOGGER_MODULE LSQLM_QDEC_HDL
 #define LSQUIC_LOG_CONN_ID lsquic_conn_log_cid(qdh->qdh_conn)
 #include "lsquic_logger.h"
 
-static void
-qdh_hblock_unblocked (void *);
+static const struct lsqpack_dec_hset_if dhi_if;
+
+
+struct header_ctx
+{
+    void                    *hset;
+    struct qpack_dec_hdl    *qdh;
+    enum ppc_flags           ppc_flags;
+    struct lsquic_ext_http_prio ehp;
+};
+
+
+/* We need to allocate struct uncompressed_headers anyway when header set
+ * is complete and we give it to the stream using lsquic_stream_uh_in().
+ * To save a malloc, we reuse context after we're done with it.
+ */
+union hblock_ctx
+{
+    struct header_ctx ctx;
+    unsigned char     space_for_uh[sizeof(struct uncompressed_headers)];
+};
 
 
 static int
@@ -128,10 +155,36 @@ lsquic_qdh_init (struct qpack_dec_hdl *qdh, struct lsquic_conn *conn,
                     int is_server, const struct lsquic_engine_public *enpub,
                     unsigned dyn_table_size, unsigned max_risked_streams)
 {
+    enum lsqpack_dec_opts dec_opts;
+
+    dec_opts = 0;
+    if (enpub->enp_hsi_if->hsi_flags & LSQUIC_HSI_HTTP1X)
+        dec_opts |= LSQPACK_DEC_OPT_HTTP1X;
+    if (enpub->enp_hsi_if->hsi_flags & LSQUIC_HSI_HASH_NAME)
+        dec_opts |= LSQPACK_DEC_OPT_HASH_NAME;
+    if (enpub->enp_hsi_if->hsi_flags & LSQUIC_HSI_HASH_NAMEVAL)
+        dec_opts |= LSQPACK_DEC_OPT_HASH_NAMEVAL;
+
+    if (conn->cn_flags & LSCONN_SERVER)
+        qdh->qdh_flags |= QDH_SERVER;
+    if (enpub->enp_settings.es_qpack_experiment)
+    {
+        qdh->qdh_exp_rec = lsquic_qpack_exp_new();
+        if (qdh->qdh_exp_rec)
+        {
+            if (conn->cn_flags & LSCONN_SERVER)
+                qdh->qdh_exp_rec->qer_flags |= QER_SERVER;
+            qdh->qdh_exp_rec->qer_used_max_size = dyn_table_size;
+            qdh->qdh_exp_rec->qer_used_max_blocked = max_risked_streams;
+        }
+    }
+    if (!qdh->qdh_exp_rec && LSQ_LOG_ENABLED_EXT(LSQ_LOG_NOTICE, LSQLM_CONN))
+        qdh->qdh_flags |= QDH_SAVE_UA;
+
     qdh->qdh_conn = conn;
     lsquic_frab_list_init(&qdh->qdh_fral, 0x400, NULL, NULL, NULL);
     lsqpack_dec_init(&qdh->qdh_decoder, (void *) conn, dyn_table_size,
-                        max_risked_streams, qdh_hblock_unblocked);
+                        max_risked_streams, &dhi_if, dec_opts);
     qdh->qdh_flags |= QDH_INITIALIZED;
     qdh->qdh_enpub = enpub;
     if (qdh->qdh_enpub->enp_hsi_if == lsquic_http1x_if)
@@ -154,12 +207,34 @@ lsquic_qdh_init (struct qpack_dec_hdl *qdh, struct lsquic_conn *conn,
 }
 
 
+static void
+qdh_log_and_clean_exp_rec (struct qpack_dec_hdl *qdh)
+{
+    char buf[0x400];
+
+    qdh->qdh_exp_rec->qer_comp_ratio = lsqpack_dec_ratio(&qdh->qdh_decoder);
+    /* Naughty: poking inside the decoder, it's not exposed.  (Should it be?) */
+    qdh->qdh_exp_rec->qer_peer_max_size = qdh->qdh_decoder.qpd_cur_max_capacity;
+    (void) lsquic_qpack_exp_to_xml(qdh->qdh_exp_rec, buf, sizeof(buf));
+    LSQ_NOTICE("%s", buf);
+    lsquic_qpack_exp_destroy(qdh->qdh_exp_rec);
+    qdh->qdh_exp_rec = NULL;
+}
+
+
 void
 lsquic_qdh_cleanup (struct qpack_dec_hdl *qdh)
 {
     if (qdh->qdh_flags & QDH_INITIALIZED)
     {
         LSQ_DEBUG("cleanup");
+        if (qdh->qdh_exp_rec)
+            qdh_log_and_clean_exp_rec(qdh);
+        if (qdh->qdh_ua)
+        {
+            free(qdh->qdh_ua);
+            qdh->qdh_ua = NULL;
+        }
         lsqpack_dec_cleanup(&qdh->qdh_decoder);
         lsquic_frab_list_cleanup(&qdh->qdh_fral);
         qdh->qdh_flags &= ~QDH_INITIALIZED;
@@ -377,7 +452,8 @@ static void
 qdh_hblock_unblocked (void *stream_p)
 {
     struct lsquic_stream *const stream = stream_p;
-    struct qpack_dec_hdl *const qdh = lsquic_stream_get_qdh(stream);
+    union hblock_ctx *const u = stream->sm_hblock_ctx;
+    struct qpack_dec_hdl *qdh = u->ctx.qdh;
 
     LSQ_DEBUG("header block for stream %"PRIu64" unblocked", stream->id);
     lsquic_stream_qdec_unblocked(stream);
@@ -432,105 +508,105 @@ process_content_length (const struct qpack_dec_hdl *qdh /* for logging */,
 
 
 static int
-is_content_length (const struct lsqpack_header *header)
+is_content_length (const struct lsxpack_header *xhdr)
 {
-    return ((header->qh_flags & QH_ID_SET) && header->qh_static_id == 4)
-        || (header->qh_name_len == 14 && header->qh_name[0] == 'c'
-                    && 0 == memcmp(header->qh_name + 1, "ontent-length", 13))
+    return ((xhdr->flags & LSXPACK_QPACK_IDX)
+                        && xhdr->qpack_index == LSQPACK_TNV_CONTENT_LENGTH_0)
+        || (xhdr->name_len == 14 && 0 == memcmp(lsxpack_header_get_name(xhdr),
+                                                        "content-length", 13))
         ;
 }
 
 
 static int
-qdh_supply_hset_to_stream (struct qpack_dec_hdl *qdh,
-            struct lsquic_stream *stream, struct lsqpack_header_list *qlist)
+is_priority (const struct lsxpack_header *xhdr)
 {
-    const struct lsquic_hset_if *const hset_if = qdh->qdh_enpub->enp_hsi_if;
-    const unsigned hpack_static_table_size = 61;
-    struct uncompressed_headers *uh = NULL;
-    const struct lsqpack_header *header;
-    enum lsquic_header_status st;
-    int push_promise;
-    unsigned i;
-    void *hset;
-    struct cont_len cl;
-
-    push_promise = lsquic_stream_header_is_pp(stream);
-    hset = hset_if->hsi_create_header_set(qdh->qdh_hsi_ctx, push_promise);
-    if (!hset)
-    {
-        LSQ_INFO("call to hsi_create_header_set failed");
-        return -1;
-    }
-
-    LSQ_DEBUG("got header set for stream %"PRIu64, stream->id);
-
-    cl.has = 0;
-    for (i = 0; i < qlist->qhl_count; ++i)
-    {
-        header = qlist->qhl_headers[i];
-        LSQ_DEBUG("%.*s: %.*s", header->qh_name_len, header->qh_name,
-                                        header->qh_value_len, header->qh_value);
-        st = hset_if->hsi_process_header(hset,
-                    header->qh_flags & QH_ID_SET ?
-                        hpack_static_table_size + 1 + header->qh_static_id : 0,
-                    header->qh_name, header->qh_name_len,
-                    header->qh_value, header->qh_value_len);
-        if (st != LSQUIC_HDR_OK)
-        {
-            LSQ_INFO("header process returned non-OK code %u", (unsigned) st);
-            goto err;
-        }
-        if (is_content_length(header))
-            process_content_length(qdh, &cl, header->qh_value,
-                                                        header->qh_value_len);
-    }
-
-    lsqpack_dec_destroy_header_list(qlist);
-    qlist = NULL;
-    st = hset_if->hsi_process_header(hset, 0, 0, 0, 0, 0);
-    if (st != LSQUIC_HDR_OK)
-        goto err;
-
-    uh = calloc(1, sizeof(*uh));
-    if (!uh)
-        goto err;
-    uh->uh_stream_id = stream->id;
-    uh->uh_oth_stream_id = 0;
-    uh->uh_weight = 0;
-    uh->uh_exclusive = -1;
-    if (hset_if == lsquic_http1x_if)
-        uh->uh_flags    |= UH_H1H;
-    uh->uh_hset = hset;
-    if (0 != lsquic_stream_uh_in(stream, uh))
-        goto err;
-    LSQ_DEBUG("converted qlist to hset and gave it to stream %"PRIu64,
-                                                                stream->id);
-    if (cl.has > 0)
-        (void) lsquic_stream_verify_len(stream, cl.value);
-    return 0;
-
-  err:
-    if (qlist)
-        lsqpack_dec_destroy_header_list(qlist);
-    hset_if->hsi_discard_header_set(hset);
-    free(uh);
-    return -1;
+    return xhdr->name_len == 8
+        && 0 == memcmp(lsxpack_header_get_name(xhdr), "priority", 8);
 }
 
 
-/* Releases qlist */
-static int
-qdh_process_qlist (struct qpack_dec_hdl *qdh,
-            struct lsquic_stream *stream, struct lsqpack_header_list *qlist)
+static struct lsxpack_header *
+qdh_prepare_decode (void *stream_p, struct lsxpack_header *xhdr, size_t space)
 {
-    if (!lsquic_stream_header_is_trailer(stream))
-        return qdh_supply_hset_to_stream(qdh, stream, qlist);
-    else
+    struct lsquic_stream *const stream = stream_p;
+    union hblock_ctx *const u = stream->sm_hblock_ctx;
+    struct qpack_dec_hdl *const qdh = u->ctx.qdh;
+
+    return qdh->qdh_enpub->enp_hsi_if->hsi_prepare_decode(
+                                                u->ctx.hset, xhdr, space);
+}
+
+
+static void
+qdh_maybe_set_user_agent (struct qpack_dec_hdl *qdh,
+                                const struct lsxpack_header *xhdr, char **ua)
+{
+    /* Flipped: we are the *decoder* */
+    const char *const name = qdh->qdh_flags & QDH_SERVER ?
+                                    "user-agent" : "server";
+    const size_t len = qdh->qdh_flags & QDH_SERVER ? 10 : 6;
+
+    if (len == xhdr->name_len
+                && 0 == memcmp(name, lsxpack_header_get_name(xhdr), len))
+        *ua = strndup(lsxpack_header_get_value(xhdr), xhdr->val_len);
+}
+
+
+static int
+qdh_process_header (void *stream_p, struct lsxpack_header *xhdr)
+{
+    struct lsquic_stream *const stream = stream_p;
+    union hblock_ctx *const u = stream->sm_hblock_ctx;
+    struct qpack_dec_hdl *const qdh = u->ctx.qdh;
+    struct cont_len cl;
+
+    if (is_content_length(xhdr))
     {
-        LSQ_DEBUG("discard trailer header set");
-        lsqpack_dec_destroy_header_list(qlist);
-        return 0;
+        cl.has = 0;
+        process_content_length(qdh, &cl, lsxpack_header_get_value(xhdr),
+                                                            xhdr->val_len);
+        if (cl.has > 0)
+            (void) lsquic_stream_verify_len(stream, cl.value);
+    }
+    else if ((stream->sm_bflags & (SMBF_HTTP_PRIO|SMBF_HPRIO_SET))
+                                                            == SMBF_HTTP_PRIO
+            && is_priority(xhdr))
+    {
+        u->ctx.ppc_flags &= ~(PPC_INC_NAME|PPC_URG_NAME);
+        (void) lsquic_http_parse_pfv(lsxpack_header_get_value(xhdr),
+                        xhdr->val_len, &u->ctx.ppc_flags, &u->ctx.ehp,
+                        (char *) stream->conn_pub->mm->acki,
+                        sizeof(*stream->conn_pub->mm->acki));
+    }
+    else if (qdh->qdh_exp_rec && !qdh->qdh_exp_rec->qer_user_agent)
+        qdh_maybe_set_user_agent(qdh, xhdr, &qdh->qdh_exp_rec->qer_user_agent);
+    else if ((qdh->qdh_flags & QDH_SAVE_UA) && !qdh->qdh_ua)
+        qdh_maybe_set_user_agent(qdh, xhdr, &qdh->qdh_ua);
+
+    return qdh->qdh_enpub->enp_hsi_if->hsi_process_header(u->ctx.hset, xhdr);
+}
+
+
+static const struct lsqpack_dec_hset_if dhi_if =
+{
+    .dhi_unblocked      = qdh_hblock_unblocked,
+    .dhi_prepare_decode = qdh_prepare_decode,
+    .dhi_process_header = qdh_process_header,
+};
+
+
+static void
+qdh_maybe_destroy_hblock_ctx (struct qpack_dec_hdl *qdh,
+                                                struct lsquic_stream *stream)
+{
+    if (stream->sm_hblock_ctx)
+    {
+        LSQ_DEBUG("destroy hblock_ctx of stream %"PRIu64, stream->id);
+        qdh->qdh_enpub->enp_hsi_if->hsi_discard_header_set(
+                                            stream->sm_hblock_ctx->ctx.hset);
+        free(stream->sm_hblock_ctx);
+        stream->sm_hblock_ctx = NULL;
     }
 }
 
@@ -538,37 +614,73 @@ qdh_process_qlist (struct qpack_dec_hdl *qdh,
 static enum lsqpack_read_header_status
 qdh_header_read_results (struct qpack_dec_hdl *qdh,
         struct lsquic_stream *stream, enum lsqpack_read_header_status rhs,
-        struct lsqpack_header_list *qlist, const unsigned char *dec_buf,
-        size_t dec_buf_sz)
+        const unsigned char *dec_buf, size_t dec_buf_sz)
 {
     const struct lsqpack_dec_err *qerr;
+    struct uncompressed_headers *uh;
+    void *hset;
 
     if (rhs == LQRHS_DONE)
     {
-        if (qlist)
+        if (!lsquic_stream_header_is_trailer(stream))
         {
-            if (0 != qdh_process_qlist(qdh, stream, qlist))
-                return LQRHS_ERROR;
-            if (qdh->qdh_dec_sm_out)
+            if (stream->sm_hblock_ctx->ctx.ppc_flags
+                                                & (PPC_INC_SET|PPC_URG_SET))
             {
-                if (dec_buf_sz
-                    && 0 != qdh_write_decoder(qdh, dec_buf, dec_buf_sz))
-                {
-                    return LQRHS_ERROR;
-                }
-                if (dec_buf_sz || lsqpack_dec_ici_pending(&qdh->qdh_decoder))
-                    lsquic_stream_wantwrite(qdh->qdh_dec_sm_out, 1);
+                assert(stream->sm_bflags & SMBF_HTTP_PRIO);
+                LSQ_DEBUG("Apply Priority from headers to stream %"PRIu64,
+                                                                stream->id);
+                (void) lsquic_stream_set_http_prio(stream,
+                                            &stream->sm_hblock_ctx->ctx.ehp);
+            }
+            hset = stream->sm_hblock_ctx->ctx.hset;
+            uh = (void *) stream->sm_hblock_ctx;
+            stream->sm_hblock_ctx = NULL;
+            memset(uh, 0, sizeof(*uh));
+            uh->uh_stream_id = stream->id;
+            uh->uh_oth_stream_id = 0;
+            uh->uh_weight = 0;
+            uh->uh_exclusive = -1;
+            if (qdh->qdh_enpub->enp_hsi_if == lsquic_http1x_if)
+                uh->uh_flags    |= UH_H1H;
+            if (0 != qdh->qdh_enpub->enp_hsi_if
+                                        ->hsi_process_header(hset, NULL))
+            {
+                LSQ_DEBUG("finishing hset failed");
+                free(uh);
+                qdh->qdh_enpub->enp_hsi_if->hsi_discard_header_set(hset);
+                return LQRHS_ERROR;
+            }
+            uh->uh_hset = hset;
+            if (0 == lsquic_stream_uh_in(stream, uh))
+                LSQ_DEBUG("gave hset to stream %"PRIu64, stream->id);
+            else
+            {
+                LSQ_DEBUG("could not give hset to stream %"PRIu64, stream->id);
+                free(uh);
+                qdh->qdh_enpub->enp_hsi_if->hsi_discard_header_set(hset);
+                return LQRHS_ERROR;
             }
         }
         else
         {
-            LSQ_WARN("read header status is DONE but header list is not set");
-            assert(0);
-            return LQRHS_ERROR;
+            LSQ_DEBUG("discard trailer header set");
+            qdh_maybe_destroy_hblock_ctx(qdh, stream);
+        }
+        if (qdh->qdh_dec_sm_out)
+        {
+            if (dec_buf_sz
+                && 0 != qdh_write_decoder(qdh, dec_buf, dec_buf_sz))
+            {
+                return LQRHS_ERROR;
+            }
+            if (dec_buf_sz || lsqpack_dec_ici_pending(&qdh->qdh_decoder))
+                lsquic_stream_wantwrite(qdh->qdh_dec_sm_out, 1);
         }
     }
     else if (rhs == LQRHS_ERROR)
     {
+        qdh_maybe_destroy_hblock_ctx(qdh, stream);
         qerr = lsqpack_dec_get_err_info(&qdh->qdh_decoder);
         qdh->qdh_conn->cn_if->ci_abort_error(qdh->qdh_conn, 1,
             HEC_QPACK_DECOMPRESSION_FAILED, "QPACK decompression error; "
@@ -586,24 +698,62 @@ lsquic_qdh_header_in_begin (struct qpack_dec_hdl *qdh,
                         const unsigned char **buf, size_t bufsz)
 {
     enum lsqpack_read_header_status rhs;
-    struct lsqpack_header_list *qlist;
+    void *hset;
+    int is_pp;
     size_t dec_buf_sz;
+    union hblock_ctx *u;
     unsigned char dec_buf[LSQPACK_LONGEST_HEADER_ACK];
 
-    if (qdh->qdh_flags & QDH_INITIALIZED)
-    {
-        dec_buf_sz = sizeof(dec_buf);
-        rhs = lsqpack_dec_header_in(&qdh->qdh_decoder, stream, stream->id,
-                        header_size, buf, bufsz, &qlist, dec_buf, &dec_buf_sz);
-        return qdh_header_read_results(qdh, stream, rhs, qlist, dec_buf,
-                                                                dec_buf_sz);
-    }
-    else
+    assert(!(stream->stream_flags & STREAM_U_READ_DONE));
+
+    if (!(qdh->qdh_flags & QDH_INITIALIZED))
     {
         LSQ_WARN("not initialized: cannot process header block");
         return LQRHS_ERROR;
     }
 
+    u = malloc(sizeof(*u));
+    if (!u)
+    {
+        LSQ_INFO("cannot allocate hblock_ctx");
+        return LQRHS_ERROR;
+    }
+
+    is_pp = lsquic_stream_header_is_pp(stream);
+    hset = qdh->qdh_enpub->enp_hsi_if->hsi_create_header_set(
+                                          qdh->qdh_hsi_ctx, stream, is_pp);
+    if (!hset)
+    {
+        free(u);
+        LSQ_DEBUG("hsi_create_header_set failure");
+        return LQRHS_ERROR;
+    }
+
+    u->ctx.hset   = hset;
+    u->ctx.qdh    = qdh;
+    u->ctx.ppc_flags = 0;
+    u->ctx.ehp       = (struct lsquic_ext_http_prio) {
+                            .urgency     = LSQUIC_DEF_HTTP_URGENCY,
+                            .incremental = LSQUIC_DEF_HTTP_INCREMENTAL,
+    };
+    stream->sm_hblock_ctx = u;
+
+    if (qdh->qdh_exp_rec)
+    {
+        const lsquic_time_t now = lsquic_time_now();
+        if (0 == qdh->qdh_exp_rec->qer_hblock_count)
+            qdh->qdh_exp_rec->qer_first_req = now;
+        qdh->qdh_exp_rec->qer_last_req = now;
+        ++qdh->qdh_exp_rec->qer_hblock_count;
+        qdh->qdh_exp_rec->qer_hblock_size += bufsz;
+    }
+
+    dec_buf_sz = sizeof(dec_buf);
+    rhs = lsqpack_dec_header_in(&qdh->qdh_decoder, stream, stream->id,
+                    header_size, buf, bufsz, dec_buf, &dec_buf_sz);
+    if (qdh->qdh_exp_rec)
+        qdh->qdh_exp_rec->qer_peer_max_blocked += rhs == LQRHS_BLOCKED;
+    return qdh_header_read_results(qdh, stream, rhs, dec_buf, dec_buf_sz);
 }
 
 
@@ -612,17 +762,21 @@ lsquic_qdh_header_in_continue (struct qpack_dec_hdl *qdh,
         struct lsquic_stream *stream, const unsigned char **buf, size_t bufsz)
 {
     enum lsqpack_read_header_status rhs;
-    struct lsqpack_header_list *qlist;
     size_t dec_buf_sz;
     unsigned char dec_buf[LSQPACK_LONGEST_HEADER_ACK];
 
+    assert(!(stream->stream_flags & STREAM_U_READ_DONE));
+
     if (qdh->qdh_flags & QDH_INITIALIZED)
     {
+        if (qdh->qdh_exp_rec)
+            qdh->qdh_exp_rec->qer_hblock_size += bufsz;
         dec_buf_sz = sizeof(dec_buf);
         rhs = lsqpack_dec_header_read(&qdh->qdh_decoder, stream,
-                                    buf, bufsz, &qlist, dec_buf, &dec_buf_sz);
-        return qdh_header_read_results(qdh, stream, rhs, qlist, dec_buf,
-                                                                dec_buf_sz);
+                                    buf, bufsz, dec_buf, &dec_buf_sz);
+        if (qdh->qdh_exp_rec)
+            qdh->qdh_exp_rec->qer_peer_max_blocked += rhs == LQRHS_BLOCKED;
+        return qdh_header_read_results(qdh, stream, rhs, dec_buf, dec_buf_sz);
     }
     else
     {
@@ -632,7 +786,7 @@ lsquic_qdh_header_in_continue (struct qpack_dec_hdl *qdh,
 }
 
 
-void
+static void
 lsquic_qdh_unref_stream (struct qpack_dec_hdl *qdh,
                                                 struct lsquic_stream *stream)
 {
@@ -650,6 +804,11 @@ lsquic_qdh_cancel_stream (struct qpack_dec_hdl *qdh,
     ssize_t nw;
     unsigned char buf[LSQPACK_LONGEST_CANCEL];
 
+    qdh_maybe_destroy_hblock_ctx(qdh, stream);
+
+    if (!qdh->qdh_dec_sm_out)
+        return;
+
     nw = lsqpack_dec_cancel_stream(&qdh->qdh_decoder, stream, buf, sizeof(buf));
     if (nw > 0)
     {
@@ -668,6 +827,33 @@ lsquic_qdh_cancel_stream (struct qpack_dec_hdl *qdh,
 }
 
 
+void
+lsquic_qdh_cancel_stream_id (struct qpack_dec_hdl *qdh,
+                                                lsquic_stream_id_t stream_id)
+{
+    ssize_t nw;
+    unsigned char buf[LSQPACK_LONGEST_CANCEL];
+
+    if (!qdh->qdh_dec_sm_out)
+        return;
+
+    nw = lsqpack_dec_cancel_stream_id(&qdh->qdh_decoder, stream_id, buf,
+                                                                sizeof(buf));
+    if (nw > 0)
+    {
+        if (0 == qdh_write_decoder(qdh, buf, nw))
+            LSQ_DEBUG("wrote %zd-byte Cancel Stream instruction for "
+                "stream %"PRIu64" to the decoder stream", nw, stream_id);
+    }
+    else if (nw == 0)
+        LSQ_DEBUG("not generating Cancel Stream instruction for "
+            "stream %"PRIu64, stream_id);
+    else
+        LSQ_WARN("cannot generate Cancel Stream instruction for "
+            "stream %"PRIu64" -- not enough buffer space", stream_id);
+}
+
+
 int
 lsquic_qdh_arm_if_unsent (struct qpack_dec_hdl *qdh, void (*func)(void *),
                                                                     void *ctx)
@@ -687,3 +873,15 @@ lsquic_qdh_arm_if_unsent (struct qpack_dec_hdl *qdh, void (*func)(void *),
         return 1;
     }
 }
+
+
+const char *
+lsquic_qdh_get_ua (const struct qpack_dec_hdl *qdh)
+{
+    if (qdh->qdh_ua)
+        return qdh->qdh_ua;
+    else if (qdh->qdh_exp_rec && qdh->qdh_exp_rec->qer_user_agent)
+        return qdh->qdh_exp_rec->qer_user_agent;
+    else
+        return NULL;
+}
