@@ -1,4 +1,4 @@
-/* Copyright (c) 2017 - 2020 LiteSpeed Technologies Inc.  See LICENSE. */
+/* Copyright (c) 2017 - 2021 LiteSpeed Technologies Inc.  See LICENSE. */
 #include <inttypes.h>
 #include <stdint.h>
 #include <stdlib.h>
@@ -137,7 +137,7 @@ lsquic_sfcw_set_max_recv_off (struct lsquic_sfcw *fc, uint64_t max_recv_off)
     }
     else
     {
-        LSQ_WARN("flow control violation: received at offset %"PRIu64", "
+        LSQ_INFO("flow control violation: received at offset %"PRIu64", "
             "while flow control receive offset is %"PRIu64,
             max_recv_off, fc->sf_recv_off);
         return 0;
