@@ -1,4 +1,4 @@
-/* Copyright (c) 2017 - 2020 LiteSpeed Technologies Inc.  See LICENSE. */
+/* Copyright (c) 2017 - 2021 LiteSpeed Technologies Inc.  See LICENSE. */
 /*
  * lsquic_spi.c - implementation of Stream Priority Iterator.
  */
@@ -21,6 +21,9 @@
 #include "lsquic_hq.h"
 #include "lsquic_hash.h"
 #include "lsquic_stream.h"
+#include "lsquic_conn_flow.h"
+#include "lsquic_rtt.h"
+#include "lsquic_conn_public.h"
 #include "lsquic_spi.h"
 
 #define LSQUIC_LOGGER_MODULE LSQLM_SPI
@@ -51,25 +54,24 @@ add_stream_to_spi (struct stream_prio_iter *iter, lsquic_stream_t *stream)
 
 
 void
-lsquic_spi_init (struct stream_prio_iter *iter, struct lsquic_stream *first,
+lsquic_spi_init (void *iter_p, struct lsquic_stream *first,
          struct lsquic_stream *last, uintptr_t next_ptr_offset,
-         enum stream_q_flags onlist_mask, const struct lsquic_conn *conn,
+         struct lsquic_conn_public *conn_pub,
          const char *name,
          int (*filter)(void *filter_ctx, struct lsquic_stream *),
          void *filter_ctx)
 {
+    struct stream_prio_iter *const iter = iter_p;
     struct lsquic_stream *stream;
     unsigned count;
 
-    iter->spi_conn          = conn;
+    iter->spi_conn          = conn_pub->lconn;
     iter->spi_name          = name ? name : "UNSET";
     iter->spi_set[0]        = 0;
     iter->spi_set[1]        = 0;
     iter->spi_set[2]        = 0;
     iter->spi_set[3]        = 0;
-    iter->spi_onlist_mask   = onlist_mask;
     iter->spi_cur_prio      = 0;
-    iter->spi_prev_stream   = NULL;
     iter->spi_next_stream   = NULL;
     iter->spi_n_added       = 0;
 
@@ -195,41 +197,13 @@ find_and_set_next_priority (struct stream_prio_iter *iter)
 }
 
 
-/* Each stream returned by the iterator is processed in some fashion.  If,
- * as a result of this, the stream gets taken off the original list, we
- * have to follow suit and remove it from the iterator's set of streams.
- */
-static void
-maybe_evict_prev (struct stream_prio_iter *iter)
-{
-    unsigned set, bit;
-
-    if (0 == (iter->spi_prev_stream->sm_qflags & iter->spi_onlist_mask))
-    {
-        SPI_DEBUG("evict stream %"PRIu64, iter->spi_prev_stream->id);
-        TAILQ_REMOVE(&iter->spi_streams[ iter->spi_prev_prio ],
-                                    iter->spi_prev_stream, next_prio_stream);
-        if (TAILQ_EMPTY(&iter->spi_streams[ iter->spi_prev_prio ]))
-        {
-            set = iter->spi_prev_prio >> 6;
-            bit = iter->spi_prev_prio & 0x3F;
-            iter->spi_set[ set ] &= ~(1ULL << bit);
-            SPI_DEBUG("priority %u now has no elements", iter->spi_prev_prio);
-        }
-        iter->spi_prev_stream = NULL;
-    }
-}
-
-
 lsquic_stream_t *
-lsquic_spi_first (struct stream_prio_iter *iter)
+lsquic_spi_first (void *iter_p)
 {
+    struct stream_prio_iter *const iter = iter_p;
     lsquic_stream_t *stream;
     unsigned set, bit;
 
-    if (iter->spi_prev_stream)
-        maybe_evict_prev(iter);
-
     iter->spi_cur_prio = 0;
     set = iter->spi_cur_prio >> 6;
     bit = iter->spi_cur_prio & 0x3F;
@@ -244,8 +218,6 @@ lsquic_spi_first (struct stream_prio_iter *iter)
     }
 
     stream = TAILQ_FIRST(&iter->spi_streams[ iter->spi_cur_prio ]);
-    iter->spi_prev_prio   = iter->spi_cur_prio;
-    iter->spi_prev_stream = stream;
     iter->spi_next_stream = TAILQ_NEXT(stream, next_prio_stream);
     if (LSQ_LOG_ENABLED(LSQ_LOG_DEBUG) && !lsquic_stream_is_critical(stream))
         SPI_DEBUG("%s: return stream %"PRIu64", priority %u", __func__,
@@ -255,18 +227,14 @@ lsquic_spi_first (struct stream_prio_iter *iter)
 
 
 lsquic_stream_t *
-lsquic_spi_next (struct stream_prio_iter *iter)
+lsquic_spi_next (void *iter_p)
 {
+    struct stream_prio_iter *const iter = iter_p;
     lsquic_stream_t *stream;
 
-    if (iter->spi_prev_stream)
-        maybe_evict_prev(iter);
-
     stream = iter->spi_next_stream;
     if (stream)
     {
-        assert(iter->spi_prev_prio == iter->spi_cur_prio);
-        iter->spi_prev_stream = stream;
         iter->spi_next_stream = TAILQ_NEXT(stream, next_prio_stream);
         if (LSQ_LOG_ENABLED(LSQ_LOG_DEBUG) && !lsquic_stream_is_critical(stream))
             SPI_DEBUG("%s: return stream %"PRIu64", priority %u", __func__,
@@ -281,8 +249,6 @@ lsquic_spi_next (struct stream_prio_iter *iter)
     }
 
     stream = TAILQ_FIRST(&iter->spi_streams[ iter->spi_cur_prio ]);
-    iter->spi_prev_prio   = iter->spi_cur_prio;
-    iter->spi_prev_stream = stream;
     iter->spi_next_stream = TAILQ_NEXT(stream, next_prio_stream);
 
     if (LSQ_LOG_ENABLED(LSQ_LOG_DEBUG) && !lsquic_stream_is_critical(stream))
@@ -343,8 +309,9 @@ spi_has_more_than_one_queue (const struct stream_prio_iter *iter)
 
 
 static void
-spi_drop_high_or_non_high (struct stream_prio_iter *iter, int drop_high)
+spi_drop_high_or_non_high (void *iter_p, int drop_high)
 {
+    struct stream_prio_iter *const iter = iter_p;
     uint64_t new_set[ sizeof(iter->spi_set) / sizeof(iter->spi_set[0]) ];
     unsigned bit, set, n;
 
@@ -376,14 +343,22 @@ spi_drop_high_or_non_high (struct stream_prio_iter *iter, int drop_high)
 
 
 void
-lsquic_spi_drop_high (struct stream_prio_iter *iter)
+lsquic_spi_drop_high (void *iter_p)
 {
+    struct stream_prio_iter *const iter = iter_p;
     spi_drop_high_or_non_high(iter, 1);
 }
 
 
 void
-lsquic_spi_drop_non_high (struct stream_prio_iter *iter)
+lsquic_spi_drop_non_high (void *iter_p)
 {
+    struct stream_prio_iter *const iter = iter_p;
     spi_drop_high_or_non_high(iter, 0);
 }
+
+
+void
+lsquic_spi_cleanup (void *iter_p)
+{
+}
