@@ -1,4 +1,4 @@
-/* Copyright (c) 2017 - 2019 LiteSpeed Technologies Inc.  See LICENSE. */
+/* Copyright (c) 2017 - 2021 LiteSpeed Technologies Inc.  See LICENSE. */
 /*
  * lsquic_stock_shi.h - Stock shared hash interface implementation.
  */
@@ -15,10 +15,10 @@ extern "C" {
 struct stock_shared_hash;
 
 struct stock_shared_hash *
-stock_shared_hash_new (void);
+lsquic_stock_shared_hash_new (void);
 
 void
-stock_shared_hash_destroy (struct stock_shared_hash *);
+lsquic_stock_shared_hash_destroy (struct stock_shared_hash *);
 
 
 #ifdef __cplusplus
