@@ -1,10 +1,7 @@
-/* Copyright (c) 2017 - 2019 LiteSpeed Technologies Inc.  See LICENSE. */
+/* Copyright (c) 2017 - 2021 LiteSpeed Technologies Inc.  See LICENSE. */
 #ifndef LSQUIC_STREAM_H
 #define LSQUIC_STREAM_H
 
-#define LSQUIC_GQUIC_STREAM_HANDSHAKE 1
-#define LSQUIC_GQUIC_STREAM_HEADERS   3
-
 #define LSQUIC_STREAM_DEFAULT_PRIO 16   /* RFC 7540, Section 5.3.5 */
 
 
@@ -19,16 +16,16 @@ struct frame_gen_ctx;
 struct data_frame;
 enum quic_frame_type;
 struct push_promise;
+union hblock_ctx;
+struct lsquic_packet_out;
+struct lsquic_send_ctl;
+struct network_path;
 
 TAILQ_HEAD(lsquic_streams_tailq, lsquic_stream);
 
 
 #ifndef LSQUIC_KEEP_STREAM_HISTORY
-#   ifdef NDEBUG
-#       define LSQUIC_KEEP_STREAM_HISTORY 0
-#   else
-#       define LSQUIC_KEEP_STREAM_HISTORY 1
-#   endif
+#   define LSQUIC_KEEP_STREAM_HISTORY 1
 #endif
 
 
@@ -87,7 +84,7 @@ struct stream_hq_frame
     }                   shf_u;
 #define shf_frame_ptr shf_u.frame_ptr
 #define shf_frame_size shf_u.frame_size
-    enum hq_frame_type  shf_frame_type:8;
+    enum hq_frame_type  shf_frame_type;
     enum shf_flags {
         SHF_TWO_BYTES   = 1 << 0,   /* Use two byte to encode frame length */
         SHF_FIXED_SIZE  = 1 << 1,   /* Payload size guaranteed */
@@ -101,12 +98,14 @@ struct stream_hq_frame
 
 struct hq_filter
 {
-    struct varint_read2_state   hqfi_vint_state;
+    struct varint_read2_state   hqfi_vint2_state;
     /* No need to copy the values: use it directly */
-#define hqfi_left hqfi_vint_state.vr2s_two
-#define hqfi_type hqfi_vint_state.vr2s_one
+#define hqfi_left hqfi_vint2_state.vr2s_two
+#define hqfi_type hqfi_vint2_state.vr2s_one
+    struct varint_read_state    hqfi_vint1_state;
+#define hqfi_push_id hqfi_vint1_state.value
     enum {
-        HQFI_FLAG_GOT_HEADERS   = 1 << 0,
+        HQFI_FLAG_UNUSED_0      = 1 << 0,
         HQFI_FLAG_ERROR         = 1 << 1,
         HQFI_FLAG_BEGIN         = 1 << 2,
         HQFI_FLAG_BLOCKED       = 1 << 3,
@@ -115,6 +114,8 @@ struct hq_filter
         HQFI_STATE_FRAME_HEADER_BEGIN,
         HQFI_STATE_FRAME_HEADER_CONTINUE,
         HQFI_STATE_READING_PAYLOAD,
+        HQFI_STATE_PUSH_ID_BEGIN,
+        HQFI_STATE_PUSH_ID_CONTINUE,
     }                           hqfi_state:8;
     unsigned char               hqfi_hist_idx;
 #define MAX_HQFI_ENTRIES (sizeof(unsigned) * 8 / 3)
@@ -147,11 +148,13 @@ enum stream_q_flags
      * connections's sending_streams queue.  Note that writing STREAM
      * frames is done separately.
      */
-#define SMQF_SENDING_FLAGS (SMQF_SEND_WUF|SMQF_SEND_RST|SMQF_SEND_BLOCKED)
+#define SMQF_SENDING_FLAGS (SMQF_SEND_WUF|SMQF_SEND_RST|SMQF_SEND_BLOCKED\
+                                                    |SMQF_SEND_STOP_SENDING)
     /* sending_streams: */
     SMQF_SEND_WUF     = 1 << 3,     /* WUF: Window Update Frame */
     SMQF_SEND_BLOCKED = 1 << 4,
     SMQF_SEND_RST     = 1 << 5,     /* Error: want to send RST_STREAM */
+    SMQF_SEND_STOP_SENDING = 1 << 10,
 
     /* The equivalent of WINDOW_UPDATE frame for streams in IETF QUIC is
      * the MAX_STREAM_DATA frame.  Define an alias for use in the IETF
@@ -164,7 +167,11 @@ enum stream_q_flags
     SMQF_FREE_STREAM  = 1 << 7,
     SMQF_ABORT_CONN   = 1 << 8,     /* Unrecoverable error occurred */
 
-    SMQF_QPACK_DEC    = 1 << 9,     /* QPACK decoder is holding a reference to this stream */
+    SMQF_QPACK_DEC    = 1 << 9,     /* QPACK decoder handler is holding a reference to this stream */
+
+    /* The stream can reference itself, preventing its own destruction: */
+#define SMQF_SELF_FLAGS SMQF_WAIT_FIN_OFF
+    SMQF_WAIT_FIN_OFF = 1 << 11,    /* Waiting for final offset: FIN or RST */
 };
 
 
@@ -174,12 +181,27 @@ enum stream_b_flags
     SMBF_SERVER       = 1 << 0,
     SMBF_IETF         = 1 << 1,
     SMBF_USE_HEADERS  = 1 << 2,
-    SMBF_CRYPTO       = 1 << 3,
+    SMBF_CRYPTO       = 1 << 3,  /* Crypto stream: applies to both gQUIC and IETF QUIC */
     SMBF_CRITICAL     = 1 << 4,  /* This is a critical stream */
     SMBF_AUTOSWITCH   = 1 << 5,
     SMBF_RW_ONCE      = 1 << 6,  /* When set, read/write events are dispatched once per call */
     SMBF_CONN_LIMITED = 1 << 7,
-#define N_SMBF_FLAGS 8
+    SMBF_HEADERS      = 1 << 8,  /* Headers stream */
+    SMBF_VERIFY_CL    = 1 << 9,  /* Verify content-length (stored in sm_cont_len) */
+    SMBF_HTTP_PRIO    = 1 <<10,  /* Extensible HTTP Priorities are used */
+    SMBF_INCREMENTAL  = 1 <<11,  /* Value of the "incremental" HTTP Priority parameter */
+    SMBF_HPRIO_SET    = 1 <<12,  /* Extensible HTTP Priorities have been set once */
+    SMBF_DELAY_ONCLOSE= 1 <<13,  /* Delay calling on_close() until peer ACKs everything */
+#define N_SMBF_FLAGS 14
+};
+
+
+/* Stream "callback done" flags */
+/* TODO: move STREAM.*DONE flags from stream_flags here */
+enum stream_d_flags
+{
+    SMDF_ONRESET0       =   1 << 0, /* Called on_reset(0) */
+    SMDF_ONRESET1       =   1 << 1, /* Called on_reset(1) */
 };
 
 
@@ -194,18 +216,18 @@ enum stream_flags {
     STREAM_FIN_REACHED  = 1 << 7,   /* User read data up to FIN */
     STREAM_FINISHED     = 1 << 8,   /* Stream is finished */
     STREAM_ONCLOSE_DONE = 1 << 9,   /* on_close has been called */
-    STREAM_UNUSED10     = 1 << 10,  /* Unused */
+    STREAM_CACHED_FRAME = 1 << 10,  /* If set, sm_has_frame can be used */
     STREAM_HEADERS_SENT = 1 << 11,
     STREAM_HAVE_UH      = 1 << 12,  /* Have uncompressed headers */
-    STREAM_UNUSED13     = 1 << 13,  /* Unused */
+    STREAM_ENCODER_DEP  = 1 << 13,  /* Encoder dependency: flush (IETF only) */
     STREAM_HEAD_IN_FIN  = 1 << 14,  /* Incoming headers has FIN bit set */
     STREAM_FRAMES_ELIDED= 1 << 15,
     STREAM_FORCE_FINISH = 1 << 16,  /* Replaces FIN sent and received */
     STREAM_ONNEW_DONE   = 1 << 17,  /* on_new_stream has been called */
     STREAM_PUSHING      = 1 << 18,
     STREAM_NOPUSH       = 1 << 19,  /* Disallow further push promises */
-    STREAM_UNUSED20     = 1 << 20,  /* Unused */
-    STREAM_UNUSED21     = 1 << 21,  /* Unused */
+    STREAM_GOAWAY_IN    = 1 << 20,  /* Incoming GOAWAY has been processed */
+    STREAM_SS_SENT      = 1 << 21,  /* STOP_SENDING sent */
     STREAM_RST_ACKED    = 1 << 22,  /* Packet containing RST has been acked */
     STREAM_BLOCKED_SENT = 1 << 23,  /* Stays set once a STREAM_BLOCKED frame is sent */
     STREAM_RST_READ     = 1 << 24,  /* User code collected the error */
@@ -262,8 +284,11 @@ struct lsquic_stream
 
     struct hq_filter                sm_hq_filter;
 
+    /* Optional tap for pwritev undo */
+    struct hq_arr                  *sm_hq_arr;
+
     /* We can safely use sm_hq_filter */
-#define sm_uni_type_state sm_hq_filter.hqfi_vint_state.vr2s_varint_state
+#define sm_uni_type_state sm_hq_filter.hqfi_vint2_state.vr2s_varint_state
 
     /** If @ref SMQF_WANT_FLUSH is set, flush until this offset. */
     uint64_t                        sm_flush_to;
@@ -280,6 +305,7 @@ struct lsquic_stream
 
     struct uncompressed_headers    *uh,
                                    *push_req;
+    union hblock_ctx               *sm_hblock_ctx;
 
     unsigned char                  *sm_buf;
     void                           *sm_onnew_arg;
@@ -298,6 +324,11 @@ struct lsquic_stream
     size_t                        (*sm_write_avail)(struct lsquic_stream *);
     int                           (*sm_readable)(struct lsquic_stream *);
 
+    struct lsquic_packet_out *    (*sm_get_packet_for_stream)(
+                                        struct lsquic_send_ctl *,
+                                        unsigned, const struct network_path *,
+                                        const struct lsquic_stream *);
+
     /* This element is optional */
     const struct stream_filter_if  *sm_sfi;
 
@@ -310,6 +341,20 @@ struct lsquic_stream
     /* Push promises sent on this stream */
     SLIST_HEAD(, push_promise)      sm_promises;
 
+    uint64_t                        sm_last_frame_off;
+
+#ifndef NDEBUG
+    /* Last time stream made progress */
+    lsquic_time_t                   sm_last_prog;
+#endif
+
+    /* Content length specified in incoming `content-length' header field.
+     * Used to verify size of DATA frames.
+     */
+    unsigned long long              sm_cont_len;
+    /* Sum of bytes in all incoming DATA frames.  Used for verification. */
+    unsigned long long              sm_data_in;
+
     /* How much data there is in sm_header_block and how much of it has been
      * sent:
      */
@@ -319,6 +364,9 @@ struct lsquic_stream
     unsigned short                  sm_n_buffered;  /* Amount of data in sm_buf */
     unsigned short                  sm_n_allocated;  /* Size of sm_buf */
 
+    /* If SMBF_HTTP_PRIO is set, the priority is used to represent the
+     * Extensible Priority urgency, which is in the range [0, 7].
+     */
     unsigned char                   sm_priority;  /* 0: high; 255: low */
     unsigned char                   sm_enc_level;
     enum {
@@ -326,11 +374,9 @@ struct lsquic_stream
         SSHS_ENC_SENDING,   /* Sending encoder stream data */
         SSHS_HBLOCK_SENDING,/* Sending header block data */
     }                               sm_send_headers_state:8;
+    enum stream_d_flags             sm_dflags:8;
     signed char                     sm_saved_want_write;
-
-    unsigned char                   sm_dup_push_off;
-    unsigned char                   sm_dup_push_len;
-    unsigned char                   sm_dup_push_buf[8];
+    signed char                     sm_has_frame;
 
 #if LSQUIC_KEEP_STREAM_HISTORY
     sm_hist_idx_t                   sm_hist_idx;
@@ -349,6 +395,7 @@ enum stream_ctor_flags
     SCF_USE_DI_HASH   = (1 << (N_SMBF_FLAGS + 1)), /* Use hash-based data input.  If not set,
                                    * the nocopy data input is used.
                                    */
+    SCF_CRYPTO_FRAMES = (1 << (N_SMBF_FLAGS + 2)), /* Write CRYPTO frames */
     SCF_DI_AUTOSWITCH = SMBF_AUTOSWITCH, /* Automatically switch between nocopy
                                    * and hash-based to data input for optimal
                                    * performance.
@@ -357,6 +404,10 @@ enum stream_ctor_flags
     SCF_CRITICAL      = SMBF_CRITICAL, /* This is a critical stream */
     SCF_IETF          = SMBF_IETF,
     SCF_HTTP          = SMBF_USE_HEADERS,
+    SCF_CRYPTO        = SMBF_CRYPTO,
+    SCF_HEADERS       = SMBF_HEADERS,
+    SCF_HTTP_PRIO     = SMBF_HTTP_PRIO,
+    SCF_DELAY_ONCLOSE = SMBF_DELAY_ONCLOSE,
 };
 
 
@@ -378,22 +429,21 @@ lsquic_stream_call_on_new (lsquic_stream_t *);
 void
 lsquic_stream_destroy (lsquic_stream_t *);
 
-/* Any of these flags will cause user-facing read and write and
- * shutdown calls to return an error.  They also make the stream
- * both readable and writeable, as we want the user to collect
- * the error.
- */
+/* True if either read or write side of the stream has been reset */
 #define lsquic_stream_is_reset(stream) \
     (((stream)->stream_flags & \
                     (STREAM_RST_RECVD|STREAM_RST_SENT|STREAM_SS_RECVD)) \
         || ((stream)->sm_qflags & SMQF_SEND_RST))
 
+int
+lsquic_stream_is_write_reset (const struct lsquic_stream *);
+
 /* Data that from the network gets inserted into the stream using
  * lsquic_stream_frame_in() function.  Returns 0 on success, -1 on
  * failure.  The latter may be caused by flow control violation or
  * invalid stream frame data, e.g. overlapping segments.
  *
- * Note that the caller does gives up control of `frame' no matter
+ * Note that the caller gives up control of `frame' no matter
  * what this function returns.
  *
  * This data is read by the user using lsquic_stream_read() function.
@@ -417,9 +467,6 @@ lsquic_stream_rst_in (lsquic_stream_t *, uint64_t offset, uint64_t error_code);
 void
 lsquic_stream_stop_sending_in (struct lsquic_stream *, uint64_t error_code);
 
-ssize_t
-lsquic_stream_read (lsquic_stream_t *stream, void *buf, size_t len);
-
 uint64_t
 lsquic_stream_read_offset (const lsquic_stream_t *stream);
 
@@ -429,11 +476,6 @@ lsquic_stream_read_offset (const lsquic_stream_t *stream);
 int
 lsquic_stream_tosend_fin (const lsquic_stream_t *stream);
 
-/* Data to be sent out to the network is written using lsquic_stream_write().
- */
-ssize_t
-lsquic_stream_write (lsquic_stream_t *stream, const void *buf, size_t len);
-
 void
 lsquic_stream_window_update (lsquic_stream_t *stream, uint64_t offset);
 
@@ -494,11 +536,9 @@ lsquic_stream_update_sfcw (lsquic_stream_t *, uint64_t max_off);
 int
 lsquic_stream_set_priority_internal (lsquic_stream_t *, unsigned priority);
 
-int
-lsquic_stream_id_is_critical (int use_http, lsquic_stream_id_t);
+#define lsquic_stream_is_critical(s) ((s)->sm_bflags & SMBF_CRITICAL)
 
-int
-lsquic_stream_is_critical (const struct lsquic_stream *);
+#define lsquic_stream_is_crypto(s) ((s)->sm_bflags & SMBF_CRYPTO)
 
 size_t
 lsquic_stream_mem_used (const struct lsquic_stream *);
@@ -522,15 +562,12 @@ size_t
 lsquic_stream_flush_threshold (const struct lsquic_stream *, unsigned);
 #endif
 
-#define crypto_level(stream) (~0ULL - (stream)->id)
+#define crypto_level(stream) (UINT64_MAX - (stream)->id)
 
 void
 lsquic_stream_set_stream_if (struct lsquic_stream *,
                    const struct lsquic_stream_if *, void *stream_if_ctx);
 
-struct qpack_dec_hdl *
-lsquic_stream_get_qdh (const struct lsquic_stream *);
-
 uint64_t
 lsquic_stream_combined_send_off (const struct lsquic_stream *);
 
@@ -578,13 +615,33 @@ lsquic_stream_qdec_unblocked (struct lsquic_stream *);
 int
 lsquic_stream_can_push (const struct lsquic_stream *);
 
-int
-lsquic_stream_duplicate_push (struct lsquic_stream *, uint64_t push_id);
-
 int
 lsquic_stream_push_promise (struct lsquic_stream *, struct push_promise *);
 
 void
 lsquic_stream_force_finish (struct lsquic_stream *);
 
+int
+lsquic_stream_header_is_pp (const struct lsquic_stream *);
+
+int
+lsquic_stream_header_is_trailer (const struct lsquic_stream *);
+
+int
+lsquic_stream_verify_len (struct lsquic_stream *, unsigned long long);
+
+#define lsquic_stream_is_blocked(stream_) ((stream_)->blocked_off && \
+                        (stream_)->blocked_off == (stream_)->max_send_off)
+
+void
+lsquic_stream_ss_frame_sent (struct lsquic_stream *);
+
+#ifndef NDEBUG
+void
+lsquic_stream_set_pwritev_params (unsigned iovecs, unsigned frames);
+#endif
+
+void
+lsquic_stream_drop_hset_ref (struct lsquic_stream *);
+
 #endif
