@@ -1306,7 +1306,7 @@ enum rop UdpListener::readOnePacket(struct read_iter *iter)
      */
     if (0 != lsquic_cid_from_packet(
                 (const unsigned char *) msg.msg_iov[0].iov_base,
-                                                msg.msg_iov[0].iov_len, &cid))
+                                                nread, &cid))
         return ROP_OK;
 
 #ifndef _NOT_USE_SHM_
@@ -1548,7 +1548,7 @@ struct ssl_ctx_st * UdpListener::getSslContext(void) const
         if (pSslContext)
             return pSslContext->get();
     }
-    
+
     return NULL;
 }
 
