@@ -53,6 +53,7 @@
 #include <stdio.h>
 #include <sys/types.h>
 #include <sys/stat.h>
+#include <time.h>
 #include <unistd.h>
 
 static char * s_pOcspCachePath = NULL;
@@ -76,7 +77,7 @@ const char *getStaplingErrMsg() { return s_ErrMsg.c_str(); }
 
 static void setLastErrMsg(const char *format, ...)
 {
-    const unsigned int MAX_LINE_LENGTH = 1024;
+    const unsigned int MAX_LINE_LENGTH = 4096;
     char s[MAX_LINE_LENGTH] = {0};
     va_list ap;
     va_start(ap, format);
@@ -114,10 +115,14 @@ int SslOcspStapling::callback(SSL *ssl)
     iResult = SSL_TLSEXT_ERR_NOACK;
     if (m_RespTime == UINT_MAX)
         return iResult;
+    unsigned char *resp_data = m_pRespData;
     update();
     if (m_pRespData && m_iDataLen > 0)
     {
-#ifndef OPENSSL_IS_BORINGSSL
+#ifdef OPENSSL_IS_BORINGSSL
+        if (m_pRespData != resp_data)
+            SSL_set_ocsp_response(ssl, m_pRespData, m_iDataLen);
+#else
         unsigned char *pOcspResp;
         /*OpenSSL will free pOcspResp by itself */
         pOcspResp = (unsigned char *)malloc(m_iDataLen);
@@ -149,7 +154,9 @@ int SslOcspStapling::processResponse(HttpFetch *pHttpFetch)
     }
     else
     {
-        setLastErrMsg("Received bad OCSP response. ReponderUrl=%s, StatusCode=%d, ContentType=%s\n",
+        setLastErrMsg("[OCSP] %s: Received bad OCSP response. ReponderUrl=%s, "
+                      "StatusCode=%d, ContentType=%s\n",
+                      m_sCertfile.c_str(),
                       m_sOcspResponder.c_str(), istatusCode,
                       ((pRespContentType) ? (pRespContentType) : ("")));
         //printf("%s\n", s_ErrMsg.c_str());
@@ -174,6 +181,7 @@ SslOcspStapling::SslOcspStapling()
     , m_pCtx(NULL)
     , m_RespTime(0)
     , m_statTime(0)
+    , m_nextUpdate(0)
     , m_pCertId(NULL)
 {
 }
@@ -205,7 +213,8 @@ int SslOcspStapling::init(SslContext *pSslCtx)
     pCert = load_cert(m_sCertfile.c_str());
     if (pCert == NULL)
     {
-        setLastErrMsg("Failed to load file: %s!\n", m_sCertfile.c_str());
+        setLastErrMsg("[OCSP] %s: Failed to load certificate file!\n",
+                      m_sCertfile.c_str());
         return -1;
     }
 
@@ -233,6 +242,22 @@ int SslOcspStapling::update()
     struct stat st;
     if (m_RespTime == UINT_MAX)
         return 0;
+    //NOTE: test code , to test clearing out expired OCSP response
+    //if (m_pRespData && m_statTime + 10 <= DateTime::s_curTime)
+    //    m_nextUpdate = DateTime::s_curTime;
+    if (m_pRespData && m_nextUpdate <= DateTime::s_curTime)
+    {
+        LS_DBG("[OCSP] %s: OCSP response expired, cur: %ld, resp_create: %ld, expire: %ld.\n",
+                    m_sRespfile.c_str(), DateTime::s_curTime, m_RespTime, m_nextUpdate);
+        releaseRespData();
+        if (m_statTime != DateTime::s_curTime)
+        {
+            m_statTime = DateTime::s_curTime;
+            goto TRY_CREATE_REQ;
+        }
+        else
+            return 0;
+    }
     if (m_statTime != 0 && m_statTime + 60 >= DateTime::s_curTime)
         return 0;
 
@@ -260,6 +285,8 @@ int SslOcspStapling::update()
             }
         }
     }
+
+TRY_CREATE_REQ:
     ret = ::stat(m_sRespfileTmp.c_str(), &st);
     if (ret != 0 || st.st_mtime + 30 < DateTime::s_curTime)
     {
@@ -314,7 +341,8 @@ int SslOcspStapling::getResponder(X509 *pCert)
         pCAt = load_cert(m_sCAfile.c_str());
         if (pCAt == NULL)
         {
-            setLastErrMsg("Failed to load file: %s!\n", m_sCAfile.c_str());
+            setLastErrMsg("[OCSP] %s: Failed to load CA file!\n",
+                          m_sCAfile.c_str());
             return -1;
         }
 
@@ -322,7 +350,8 @@ int SslOcspStapling::getResponder(X509 *pCert)
         X509_free(pCAt);
         if (strResp == NULL)
         {
-            setLastErrMsg("Failed to get responder!\n");
+            setLastErrMsg("[OCSP] %s: Failed to get responder from CA!\n",
+                          m_sCAfile.c_str());
             return -1;
         }
     }
@@ -338,7 +367,8 @@ int SslOcspStapling::getResponder(X509 *pCert)
         return 0;
     }
     X509_email_free(strResp);
-    setLastErrMsg("Failed to get responder Url!\n");
+    setLastErrMsg("[OCSP] %s: Failed to get responder Url!\n",
+                  m_sCAfile.c_str());
     return -1;
 }
 
@@ -405,7 +435,8 @@ int SslOcspStapling::createRequest()
                            (const char *)m_pReqData, len,
                            m_sRespfileTmp.c_str(), "application/ocsp-request", 
                            NULL);
-    setLastErrMsg("%p, len = %d\n", m_pHttpFetch, len);
+    LS_DBG("[OCSP] %s: %p, len = %d\n", m_sCertfile.c_str(),
+            m_pHttpFetch, len);
     //printf("%s\n", s_ErrMsg.c_str());
     return 0;
 }
@@ -449,6 +480,18 @@ int SslOcspStapling::updateRespData(OCSP_RESPONSE *pResponse)
     return -1;
 }
 
+
+time_t ASN1_TIME_to_time_t(const ASN1_TIME * asn1)
+{
+    int days = 0, seconds = 0;
+    if (!ASN1_TIME_diff(&days, &seconds, NULL, asn1))
+        return -1;
+    time_t t = time(NULL);
+    t += days * 86400 + seconds;
+    return t;
+}
+
+
 int SslOcspStapling::certVerify(OCSP_RESPONSE *pResponse,
                                 OCSP_BASICRESP *pBasicResp, X509_STORE *pXstore)
 {
@@ -499,6 +542,7 @@ int SslOcspStapling::certVerify(OCSP_RESPONSE *pResponse,
 
         if (validate == 1)
         {
+            m_nextUpdate = ASN1_TIME_to_time_t(pNextupdate);
             iResult = 0;
         }
         else
@@ -517,7 +561,8 @@ int SslOcspStapling::certVerify(OCSP_RESPONSE *pResponse,
     }
     if (iResult)
     {
-        setLastErrMsg("%s", SslError().what());
+        setLastErrMsg("[OCSP] %s: OCSP_basic_verify() error: %s",
+                      m_sCertfile.c_str(), SslError().what());
         ERR_clear_error();
         if (m_pHttpFetch)
             m_pHttpFetch->writeLog(s_ErrMsg.c_str());
@@ -643,25 +688,29 @@ int SslOcspStapling::getCertId(X509 *pCert)
     pXstore = SSL_CTX_get_cert_store(m_pCtx->get());
     if (pXstore == NULL)
     {
-        setLastErrMsg("SSL_CTX_get_cert_store failed!");
+        setLastErrMsg("[OCSP] %s: SSL_CTX_get_cert_store failed!",
+                      m_sCertfile.c_str());
         return -1;
     }
     pXstore_ctx = X509_STORE_CTX_new();
     if (pXstore_ctx == NULL)
     {
-        setLastErrMsg("X509_STORE_CTX_new failed!");
+        setLastErrMsg("[OCSP] %s: X509_STORE_CTX_new failed!",
+                      m_sCertfile.c_str());
         return -1;
     }
     if (X509_STORE_CTX_init(pXstore_ctx, pXstore, NULL, NULL) == 0)
     {
-        setLastErrMsg("X509_STORE_CTX_init failed!");
+        setLastErrMsg("[OCSP] %s: X509_STORE_CTX_init failed!",
+                      m_sCertfile.c_str());
         return -1;
     }
     n = X509_STORE_CTX_get1_issuer(&pXissuer, pXstore_ctx, pCert);
     X509_STORE_CTX_free(pXstore_ctx);
     if ((n == -1) || (n == 0))
     {
-        setLastErrMsg("X509_STORE_CTX_get1_issuer failed!");
+        setLastErrMsg("[OCSP] %s: X509_STORE_CTX_get1_issuer failed!",
+                      m_sCertfile.c_str());
         return -1;
     }
     m_pCertId = OCSP_cert_to_id(NULL, pCert, pXissuer);
